"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import tempfile

import jinja2.exceptions
import pytest
import salt.modules.debian_ip as debian_ip
import salt.utils.files
import salt.utils.platform
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

try:
    from salt.utils.odict import OrderedDict as odict
except ImportError:
    from collections import OrderedDict as odict

# Big pile of interface data for unit tests
#   To skip, search for 'DebianIpTestCase'
# fmt: off
test_interfaces = [
        # Structure
        #{'iface_name': 'ethX', 'iface_type': 'eth', 'enabled': True,
        #    'skip_test': bool(),        # True to disable this test
        #    'build_interface': dict(),  # data read from sls
        #    'get_interface(): OrderedDict(),   # data read from interfaces file
        #    'return': list()},          # jinja-rendered data

        # IPv4-only interface; single address
        {'iface_name': 'eth1', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '192.168.4.9',
                'netmask': '255.255.255.0',
                'gateway': '192.168.4.1',
                'enable_ipv6': False,
                'noifupdown': True,
                },
            'get_interface': odict([('eth1', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '192.168.4.9'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '192.168.4.1'),
                    ])),
                ]))]))]),
            'return': [
                'auto eth1\n',
                'iface eth1 inet static\n',
                '    address 192.168.4.9\n',
                '    netmask 255.255.255.0\n',
                '    gateway 192.168.4.1\n',
                '\n']},

        # IPv6-only; single address
        {'iface_name': 'eth2', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:beef::3',
                'ipv6netmask': '64',
                'ipv6gateway': '2001:db8:dead:beef::1',
                'enable_ipv6': True,
                'noifupdown': True,
                },
            'get_interface': odict([('eth2', odict([('enabled', True), ('data', odict([
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:beef::3'),
                    ('netmask', 64),
                    ('gateway', '2001:db8:dead:beef::1'),
                    ])),
                ]))]))]),
            'return': [
                'auto eth2\n',
                'iface eth2 inet6 static\n',
                '    address 2001:db8:dead:beef::3\n',
                '    netmask 64\n',
                '    gateway 2001:db8:dead:beef::1\n',
                '\n']},

        # IPv6-only; multiple addrs; no gw; first addr from ipv6addr
        {'iface_name': 'eth3', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:beef::5/64',
                'ipv6ipaddrs': [
                    '2001:db8:dead:beef::7/64',
                    '2001:db8:dead:beef::8/64',
                    '2001:db8:dead:beef::9/64'],
                'enable_ipv6': True,
                'noifupdown': True,
                },
            'get_interface': odict([('eth3', odict([('enabled', True), ('data', odict([
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:beef::5/64'),
                    ('addresses', [
                        '2001:db8:dead:beef::7/64',
                        '2001:db8:dead:beef::8/64',
                        '2001:db8:dead:beef::9/64',
                        ]),
                    ])),
                ]))]))]),
            'return': [
                'auto eth3\n',
                'iface eth3 inet6 static\n',
                '    address 2001:db8:dead:beef::5/64\n',
                '    address 2001:db8:dead:beef::7/64\n',
                '    address 2001:db8:dead:beef::8/64\n',
                '    address 2001:db8:dead:beef::9/64\n',
                '\n']},

        # IPv6-only; multiple addresses
        {'iface_name': 'eth4', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'ipv6proto': 'static',
                'ipv6ipaddrs': [
                    '2001:db8:dead:beef::5/64',
                    '2001:db8:dead:beef::7/64',
                    '2001:db8:dead:beef::8/64',
                    '2001:db8:dead:beef::9/64'],
                'ipv6gateway': '2001:db8:dead:beef::1',
                'enable_ipv6': True,
                'noifupdown': True,
                },
            'get_interface': odict([('eth4', odict([('enabled', True), ('data', odict([
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:beef::5/64'),
                    ('addresses', [
                        '2001:db8:dead:beef::7/64',
                        '2001:db8:dead:beef::8/64',
                        '2001:db8:dead:beef::9/64',
                        ]),
                    ('gateway', '2001:db8:dead:beef::1'),
                    ])),
                ]))]))]),
            'return': [
                'auto eth4\n',
                'iface eth4 inet6 static\n',
                '    address 2001:db8:dead:beef::5/64\n',
                '    address 2001:db8:dead:beef::7/64\n',
                '    address 2001:db8:dead:beef::8/64\n',
                '    address 2001:db8:dead:beef::9/64\n',
                '    gateway 2001:db8:dead:beef::1\n',
                '\n']},

        # IPv4 and IPv6 settings with v4 disabled
        {'iface_name': 'eth5', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '192.168.4.9',
                'netmask': '255.255.255.0',
                'gateway': '192.168.4.1',
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:beef::3',
                'ipv6netmask': '64',
                'ipv6gateway': '2001:db8:dead:beef::1',
                'enable_ipv4': False,
                'noifupdown': True,
                },
            'get_interface': odict([('eth5', odict([('enabled', True), ('data', odict([
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:beef::3'),
                    ('netmask', 64),
                    ('gateway', '2001:db8:dead:beef::1'),
                    ])),
                ]))]))]),
            'return': [
                'auto eth5\n',
                'iface eth5 inet6 static\n',
                '    address 2001:db8:dead:beef::3\n',
                '    netmask 64\n',
                '    gateway 2001:db8:dead:beef::1\n',
                '\n']},

        # IPv4 and IPv6 settings with v6 disabled
        {'iface_name': 'eth6', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '192.168.4.9',
                'netmask': '255.255.255.0',
                'gateway': '192.168.4.1',
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:beef::3',
                'ipv6netmask': '64',
                'ipv6gateway': '2001:db8:dead:beef::1',
                'enable_ipv6': False,
                'noifupdown': True,
                },
            'get_interface': odict([('eth6', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '192.168.4.9'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '192.168.4.1'),
                    ])),
                ]))]))]),
            'return': [
                'auto eth6\n',
                'iface eth6 inet static\n',
                '    address 192.168.4.9\n',
                '    netmask 255.255.255.0\n',
                '    gateway 192.168.4.1\n',
                '\n']},

        # IPv4 and IPv6; shared/overridden settings
        {'iface_name': 'eth7', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '192.168.4.9',
                'netmask': '255.255.255.0',
                'gateway': '192.168.4.1',
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:beef::3',
                'ipv6netmask': '64',
                'ipv6gateway': '2001:db8:dead:beef::1',
                'ttl': '18',  # shared
                'ipv6ttl': '15',  # overridden for v6
                'mtu': '1480',  # shared
                'enable_ipv6': True,
                'noifupdown': True,
                },
            'get_interface': odict([('eth7', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '192.168.4.9'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '192.168.4.1'),
                    ('ttl', 18),
                    ('mtu', 1480),
                    ])),
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:beef::3'),
                    ('netmask', 64),
                    ('gateway', '2001:db8:dead:beef::1'),
                    ('ttl', 15),
                    ('mtu', 1480),
                    ])),
                ]))]))]),
            'return': [
                'auto eth7\n',
                'iface eth7 inet static\n',
                '    address 192.168.4.9\n',
                '    netmask 255.255.255.0\n',
                '    gateway 192.168.4.1\n',
                '    ttl 18\n',
                '    mtu 1480\n',
                'iface eth7 inet6 static\n',
                '    address 2001:db8:dead:beef::3\n',
                '    netmask 64\n',
                '    gateway 2001:db8:dead:beef::1\n',
                '    ttl 15\n',
                '    mtu 1480\n',
                '\n']},

        # Slave iface
        {'iface_name': 'eth8', 'iface_type': 'slave', 'enabled': True,
            'build_interface': {
                'master': 'bond0',
                'noifupdown': True,
                },
            'get_interface': odict([('eth8', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'manual'),
                    ('filename', None),
                    ('bonding', odict([
                        ('master', 'bond0'),
                        ])),
                    ('bonding_keys', ['master']),
                    ])),
                ]))]))]),
            'return': [
                'auto eth8\n',
                'iface eth8 inet manual\n',
                '    bond-master bond0\n',
                '\n']},

        # Bond; with address IPv4 and IPv6 address; slaves as string
        {'iface_name': 'bond9', 'iface_type': 'bond', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '10.1.0.14',
                'netmask': '255.255.255.0',
                'gateway': '10.1.0.1',
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:c0::3',
                'ipv6netmask': '64',
                'ipv6gateway': '2001:db8:dead:c0::1',
                'mode': '802.3ad',
                'slaves': 'eth4 eth5',
                'enable_ipv6': True,
                'noifupdown': True,
                },
            'get_interface': odict([('bond9', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '10.1.0.14'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '10.1.0.1'),
                    ('bonding', odict([
                        ('ad_select', '0'),
                        ('downdelay', '200'),
                        ('lacp_rate', '0'),
                        ('miimon', '100'),
                        ('mode', '4'),
                        ('slaves', 'eth4 eth5'),
                        ('updelay', '0'),
                        ('use_carrier', 'on'),
                        ])),
                    ('bonding_keys', [
                        'ad_select',
                        'downdelay',
                        'lacp_rate',
                        'miimon',
                        'mode',
                        'slaves',
                        'updelay',
                        'use_carrier',
                        ]),
                    ])),
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:c0::3'),
                    ('netmask', 64),
                    ('gateway', '2001:db8:dead:c0::1'),
                    ('bonding', odict([
                        ('ad_select', '0'),
                        ('downdelay', '200'),
                        ('lacp_rate', '0'),
                        ('miimon', '100'),
                        ('mode', '4'),
                        ('slaves', 'eth4 eth5'),
                        ('updelay', '0'),
                        ('use_carrier', 'on'),
                        ])),
                    ('bonding_keys', [
                        'ad_select',
                        'downdelay',
                        'lacp_rate',
                        'miimon',
                        'mode',
                        'slaves',
                        'updelay',
                        'use_carrier',
                        ]),
                    ])),
                ]))]))]),
            'return': [
                'auto bond9\n',
                'iface bond9 inet static\n',
                '    address 10.1.0.14\n',
                '    netmask 255.255.255.0\n',
                '    gateway 10.1.0.1\n',
                '    bond-ad_select 0\n',
                '    bond-downdelay 200\n',
                '    bond-lacp_rate 0\n',
                '    bond-miimon 100\n',
                '    bond-mode 4\n',
                '    bond-slaves eth4 eth5\n',
                '    bond-updelay 0\n',
                '    bond-use_carrier on\n',
                'iface bond9 inet6 static\n',
                '    address 2001:db8:dead:c0::3\n',
                '    netmask 64\n',
                '    gateway 2001:db8:dead:c0::1\n',
                '    bond-ad_select 0\n',
                '    bond-downdelay 200\n',
                '    bond-lacp_rate 0\n',
                '    bond-miimon 100\n',
                '    bond-mode 4\n',
                '    bond-slaves eth4 eth5\n',
                '    bond-updelay 0\n',
                '    bond-use_carrier on\n',
                '\n']},

        # Bond; with address IPv4 and IPv6 address; slaves as list
        {'iface_name': 'bond10', 'iface_type': 'bond', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '10.1.0.14',
                'netmask': '255.255.255.0',
                'gateway': '10.1.0.1',
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:c0::3',
                'ipv6netmask': '64',
                'ipv6gateway': '2001:db8:dead:c0::1',
                'mode': '802.3ad',
                'slaves': ['eth4', 'eth5'],
                'enable_ipv6': True,
                'noifupdown': True,
                },
            'get_interface': odict([('bond10', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '10.1.0.14'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '10.1.0.1'),
                    ('bonding', odict([
                        ('ad_select', '0'),
                        ('downdelay', '200'),
                        ('lacp_rate', '0'),
                        ('miimon', '100'),
                        ('mode', '4'),
                        ('slaves', 'eth4 eth5'),
                        ('updelay', '0'),
                        ('use_carrier', 'on'),
                        ])),
                    ('bonding_keys', [
                        'ad_select',
                        'downdelay',
                        'lacp_rate',
                        'miimon',
                        'mode',
                        'slaves',
                        'updelay',
                        'use_carrier',
                        ]),
                    ])),
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:c0::3'),
                    ('netmask', 64),
                    ('gateway', '2001:db8:dead:c0::1'),
                    ('bonding', odict([
                        ('ad_select', '0'),
                        ('downdelay', '200'),
                        ('lacp_rate', '0'),
                        ('miimon', '100'),
                        ('mode', '4'),
                        ('slaves', 'eth4 eth5'),
                        ('updelay', '0'),
                        ('use_carrier', 'on'),
                        ])),
                    ('bonding_keys', [
                        'ad_select',
                        'downdelay',
                        'lacp_rate',
                        'miimon',
                        'mode',
                        'slaves',
                        'updelay',
                        'use_carrier',
                        ]),
                    ])),
                ]))]))]),
            'return': [
                'auto bond10\n',
                'iface bond10 inet static\n',
                '    address 10.1.0.14\n',
                '    netmask 255.255.255.0\n',
                '    gateway 10.1.0.1\n',
                '    bond-ad_select 0\n',
                '    bond-downdelay 200\n',
                '    bond-lacp_rate 0\n',
                '    bond-miimon 100\n',
                '    bond-mode 4\n',
                '    bond-slaves eth4 eth5\n',
                '    bond-updelay 0\n',
                '    bond-use_carrier on\n',
                'iface bond10 inet6 static\n',
                '    address 2001:db8:dead:c0::3\n',
                '    netmask 64\n',
                '    gateway 2001:db8:dead:c0::1\n',
                '    bond-ad_select 0\n',
                '    bond-downdelay 200\n',
                '    bond-lacp_rate 0\n',
                '    bond-miimon 100\n',
                '    bond-mode 4\n',
                '    bond-slaves eth4 eth5\n',
                '    bond-updelay 0\n',
                '    bond-use_carrier on\n',
                '\n']},

        # Bond VLAN; with IPv4 address
        {'iface_name': 'bond0.11', 'iface_type': 'vlan', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '10.7.0.8',
                'netmask': '255.255.255.0',
                'gateway': '10.7.0.1',
                'slaves': 'eth6 eth7',
                'mode': '802.3ad',
                'enable_ipv6': False,
                'noifupdown': True,
                },
            'get_interface': odict([('bond0.11', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('vlan_raw_device', 'bond1'),
                    ('address', '10.7.0.8'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '10.7.0.1'),
                    ('mode', '802.3ad'),
                    ])),
                ]))]))]),
            'return': [
                'auto bond0.11\n',
                'iface bond0.11 inet static\n',
                '    vlan-raw-device bond1\n',
                '    address 10.7.0.8\n',
                '    netmask 255.255.255.0\n',
                '    gateway 10.7.0.1\n',
                '    mode 802.3ad\n',
                '\n']},

        # Bond; without address
        {'iface_name': 'bond0.12', 'iface_type': 'vlan', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'slaves': 'eth6 eth7',
                'mode': '802.3ad',
                'enable_ipv6': False,
                'noifupdown': True,
                },
            'get_interface': odict([('bond0.12', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('vlan_raw_device', 'bond1'),
                    ('mode', '802.3ad'),
                    ])),
                ]))]))]),
            'return': [
                'auto bond0.12\n',
                'iface bond0.12 inet static\n',
                '    vlan-raw-device bond1\n',
                '    mode 802.3ad\n',
                '\n']},

        # DNS NS as list
        {'iface_name': 'eth13', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '192.168.4.9',
                'netmask': '255.255.255.0',
                'gateway': '192.168.4.1',
                'enable_ipv6': False,
                'noifupdown': True,
                'dns': ['8.8.8.8', '8.8.4.4'],
                },
            'get_interface': odict([('eth13', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '192.168.4.9'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '192.168.4.1'),
                    ('dns_nameservers', ['8.8.8.8', '8.8.4.4']),
                    ])),
                ]))]))]),
            'return': [
                'auto eth13\n',
                'iface eth13 inet static\n',
                '    address 192.168.4.9\n',
                '    netmask 255.255.255.0\n',
                '    gateway 192.168.4.1\n',
                '    dns-nameservers 8.8.8.8 8.8.4.4\n',
                '\n']},

        # DNS NS as string
        {'iface_name': 'eth14', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'static',
                'ipaddr': '192.168.4.9',
                'netmask': '255.255.255.0',
                'gateway': '192.168.4.1',
                'enable_ipv6': False,
                'noifupdown': True,
                'dns': '8.8.8.8 8.8.4.4',
                },
            'get_interface': odict([('eth14', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '192.168.4.9'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '192.168.4.1'),
                    ('dns_nameservers', ['8.8.8.8', '8.8.4.4']),
                    ])),
                ]))]))]),
            'return': [
                'auto eth14\n',
                'iface eth14 inet static\n',
                '    address 192.168.4.9\n',
                '    netmask 255.255.255.0\n',
                '    gateway 192.168.4.1\n',
                '    dns-nameservers 8.8.8.8 8.8.4.4\n',
                '\n']},

        # Loopback; with IPv4 and IPv6 address
        {'iface_name': 'lo15', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'loopback',
                'ipaddr': '192.168.4.9',
                'netmask': '255.255.255.0',
                'gateway': '192.168.4.1',
                'enable_ipv6': True,
                'ipv6proto': 'loopback',
                'ipv6ipaddr': 'fc00::1',
                'ipv6netmask': '128',
                'ipv6_autoconf': False,
                'noifupdown': True,
                },
            'get_interface': odict([('lo15', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'loopback'),
                    ('filename', None),
                    ('address', '192.168.4.9'),
                    ('netmask', '255.255.255.0'),
                    ('gateway', '192.168.4.1'),
                    ])),
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'loopback'),
                    ('filename', None),
                    ('address', 'fc00::1'),
                    ('netmask', 128),
                    ])),
                ]))]))]),
            'return': [
                'auto lo15\n',
                'iface lo15 inet loopback\n',
                '    address 192.168.4.9\n',
                '    netmask 255.255.255.0\n',
                '    gateway 192.168.4.1\n',
                'iface lo15 inet6 loopback\n',
                '    address fc00::1\n',
                '    netmask 128\n',
                '\n']},

        # Loopback; with only IPv6 address; enabled=False
        {'iface_name': 'lo16', 'iface_type': 'eth', 'enabled': False,
            'build_interface': {
                'enable_ipv6': True,
                'ipv6proto': 'loopback',
                'ipv6ipaddr': 'fc00::1',
                'ipv6netmask': '128',
                'ipv6_autoconf': False,
                'noifupdown': True,
                },
            'get_interface': odict([('lo16', odict([('data', odict([
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'loopback'),
                    ('filename', None),
                    ('address', 'fc00::1'),
                    ('netmask', 128),
                    ])),
                ]))]))]),
            'return': [
                'iface lo16 inet6 loopback\n',
                '    address fc00::1\n',
                '    netmask 128\n',
                '\n']},

        # Loopback; without address
        {'iface_name': 'lo17', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'loopback',
                'enable_ipv6': False,
                'noifupdown': True,
                },
            'get_interface': odict([('lo17', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'loopback'),
                    ('filename', None),
                    ])),
                ]))]))]),
            'return': [
                'auto lo17\n',
                'iface lo17 inet loopback\n',
                '\n']},

        # IPv4=DHCP; IPv6=Static; with IPv6 netmask
        {'iface_name': 'eth18', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'dhcp',
                'enable_ipv6': True,
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:c0::3',
                'ipv6netmask': '64',
                'ipv6gateway': '2001:db8:dead:c0::1',
                'noifupdown': True,
                },
            'get_interface': odict([('eth18', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'dhcp'),
                    ('filename', None),
                    ])),
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:c0::3'),
                    ('netmask', 64),
                    ('gateway', '2001:db8:dead:c0::1'),
                    ])),
                ]))]))]),
            'return': [
                'auto eth18\n',
                'iface eth18 inet dhcp\n',
                'iface eth18 inet6 static\n',
                '    address 2001:db8:dead:c0::3\n',
                '    netmask 64\n',
                '    gateway 2001:db8:dead:c0::1\n',
                '\n']},

        # IPv4=DHCP; IPv6=Static; without IPv6 netmask
        {'iface_name': 'eth19', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'proto': 'dhcp',
                'enable_ipv6': True,
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:c0::3/64',
                'ipv6gateway': '2001:db8:dead:c0::1',
                'noifupdown': True,
                },
            'get_interface': odict([('eth19', odict([('enabled', True), ('data', odict([
                ('inet', odict([
                    ('addrfam', 'inet'),
                    ('proto', 'dhcp'),
                    ('filename', None),
                    ])),
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('address', '2001:db8:dead:c0::3/64'),
                    ('gateway', '2001:db8:dead:c0::1'),
                    ])),
                ]))]))]),
            'return': [
                'auto eth19\n',
                'iface eth19 inet dhcp\n',
                'iface eth19 inet6 static\n',
                '    address 2001:db8:dead:c0::3/64\n',
                '    gateway 2001:db8:dead:c0::1\n',
                '\n']},

        # IPv6-only; static with autoconf and accept_ra forced
        {'iface_name': 'eth20', 'iface_type': 'eth', 'enabled': True,
            'build_interface': {
                'ipv6proto': 'static',
                'ipv6ipaddr': '2001:db8:dead:beef::3/64',
                'ipv6gateway': '2001:db8:dead:beef::1',
                'enable_ipv6': True,
                'autoconf': 1,
                'accept_ra': 2,
                'noifupdown': True,
                },
            'get_interface': odict([('eth20', odict([('enabled', True), ('data', odict([
                ('inet6', odict([
                    ('addrfam', 'inet6'),
                    ('proto', 'static'),
                    ('filename', None),
                    ('autoconf', 1),
                    ('address', '2001:db8:dead:beef::3/64'),
                    ('gateway', '2001:db8:dead:beef::1'),
                    ('accept_ra', 2),
                    ])),
                ]))]))]),
            'return': [
                'auto eth20\n',
                'iface eth20 inet6 static\n',
                '    autoconf 1\n',
                '    address 2001:db8:dead:beef::3/64\n',
                '    gateway 2001:db8:dead:beef::1\n',
                '    accept_ra 2\n',
                '\n']},
        ]
# fmt: on


@skipIf(salt.utils.platform.is_windows(), "Do not run these tests on Windows")
class DebianIpTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.debian_ip
    """

    def setup_loader_modules(self):
        return {debian_ip: {}}

    # 'build_bond' function tests: 3

    def test_build_bond(self):
        """
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        """
        with patch(
            "salt.modules.debian_ip._parse_settings_bond", MagicMock(return_value={})
        ), patch("salt.modules.debian_ip._write_file", MagicMock(return_value=True)):
            mock = MagicMock(return_value=1)
            with patch.dict(debian_ip.__grains__, {"osrelease": mock}):
                mock = MagicMock(return_value=True)
                with patch.dict(
                    debian_ip.__salt__, {"kmod.load": mock, "pkg.install": mock}
                ):
                    self.assertEqual(debian_ip.build_bond("bond0"), "")

    def test_error_message_iface_should_process_non_str_expected(self):
        values = [1, True, False, "no-kaboom"]
        iface = "ethtest"
        option = "test"
        msg = debian_ip._error_msg_iface(iface, option, values)
        self.assertTrue(msg.endswith("[1|True|False|no-kaboom]"), msg)

    def test_error_message_network_should_process_non_str_expected(self):
        values = [1, True, False, "no-kaboom"]
        msg = debian_ip._error_msg_network("fnord", values)
        self.assertTrue(msg.endswith("[1|True|False|no-kaboom]"), msg)

    def test_build_bond_exception(self):
        """
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        """
        with patch(
            "salt.modules.debian_ip._parse_settings_bond", MagicMock(return_value={})
        ):
            mock = MagicMock(return_value=1)
            with patch.dict(debian_ip.__grains__, {"osrelease": mock}):
                mock = MagicMock(
                    side_effect=jinja2.exceptions.TemplateNotFound("error")
                )
                with patch.object(jinja2.Environment, "get_template", mock):
                    self.assertEqual(debian_ip.build_bond("bond0"), "")

    def test_build_bond_data(self):
        """
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        """
        with patch(
            "salt.modules.debian_ip._parse_settings_bond", MagicMock(return_value={})
        ), patch("salt.modules.debian_ip._read_temp", MagicMock(return_value=True)):
            mock = MagicMock(return_value=1)
            with patch.dict(debian_ip.__grains__, {"osrelease": mock}):
                self.assertTrue(debian_ip.build_bond("bond0", test="True"))

    # 'build_routes' function tests: 2

    def test_build_routes(self):
        """
        Test if it add route scripts for a network interface using up commands.
        """
        with patch(
            "salt.modules.debian_ip._parse_routes",
            MagicMock(return_value={"routes": []}),
        ), patch(
            "salt.modules.debian_ip._write_file_routes", MagicMock(return_value=True)
        ), patch(
            "salt.modules.debian_ip._read_file", MagicMock(return_value="salt")
        ):
            self.assertEqual(debian_ip.build_routes("eth0"), "saltsalt")

    def test_build_routes_exception(self):
        """
        Test if it add route scripts for a network interface using up commands.
        """
        with patch(
            "salt.modules.debian_ip._parse_routes",
            MagicMock(return_value={"routes": []}),
        ):
            self.assertTrue(debian_ip.build_routes("eth0", test="True"))

            mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound("err"))
            with patch.object(jinja2.Environment, "get_template", mock):
                self.assertEqual(debian_ip.build_routes("eth0"), "")

    # 'down' function tests: 1

    def test_down(self):
        """
        Test if it shutdown a network interface
        """
        self.assertEqual(debian_ip.down("eth0", "slave"), None)

        mock = MagicMock(return_value="Salt")
        with patch.dict(debian_ip.__salt__, {"cmd.run": mock}):
            self.assertEqual(debian_ip.down("eth0", "eth"), "Salt")

    # 'get_bond' function tests: 1

    def test_get_bond(self):
        """
        Test if it return the content of a bond script
        """
        self.assertEqual(debian_ip.get_bond("bond0"), "")

    # '_parse_interfaces' function tests: 1

    def test_parse_interfaces(self):
        """
        Test if it returns the correct data for parsed configuration file
        """
        with tempfile.NamedTemporaryFile(mode="r", delete=True) as tfile:
            for iface in test_interfaces:
                iname = iface["iface_name"]
                if iface.get("skip_test", False):
                    continue
                with salt.utils.files.fopen(str(tfile.name), "w") as fh:
                    fh.writelines(iface["return"])
                for inet in ["inet", "inet6"]:
                    if inet in iface["get_interface"][iname]["data"]:
                        iface["get_interface"][iname]["data"][inet]["filename"] = str(
                            tfile.name
                        )
                self.assertDictEqual(
                    debian_ip._parse_interfaces([str(tfile.name)]),
                    iface["get_interface"],
                )

    # 'get_interface' function tests: 1

    def test_get_interface(self):
        """
        Test if it return the contents of an interface script
        """
        for iface in test_interfaces:
            if iface.get("skip_test", False):
                continue
            with patch.object(
                debian_ip,
                "_parse_interfaces",
                MagicMock(return_value=iface["get_interface"]),
            ):
                self.assertListEqual(
                    debian_ip.get_interface(iface["iface_name"]), iface["return"]
                )

    # 'build_interface' function tests: 1

    def test_build_interface(self):
        """
        Test if it builds an interface script for a network interface.
        """
        with patch(
            "salt.modules.debian_ip._write_file_ifaces", MagicMock(return_value="salt")
        ):
            self.assertEqual(
                debian_ip.build_interface("eth0", "eth", "enabled"),
                ["s\n", "a\n", "l\n", "t\n"],
            )

            self.assertTrue(
                debian_ip.build_interface("eth0", "eth", "enabled", test="True")
            )

            with patch.object(
                debian_ip, "_parse_settings_eth", MagicMock(return_value={"routes": []})
            ):
                for eth_t in ["bridge", "slave", "bond"]:
                    self.assertRaises(
                        AttributeError,
                        debian_ip.build_interface,
                        "eth0",
                        eth_t,
                        "enabled",
                    )

            self.assertTrue(
                debian_ip.build_interface("eth0", "eth", "enabled", test="True")
            )

        with tempfile.NamedTemporaryFile(mode="r", delete=True) as tfile:
            with patch("salt.modules.debian_ip._DEB_NETWORK_FILE", str(tfile.name)):
                for iface in test_interfaces:
                    if iface.get("skip_test", False):
                        continue
                    # Skip tests that require __salt__['pkg.install']()
                    if iface["iface_type"] in ["bridge", "pppoe", "vlan"]:
                        continue
                    self.assertListEqual(
                        debian_ip.build_interface(
                            iface=iface["iface_name"],
                            iface_type=iface["iface_type"],
                            enabled=iface["enabled"],
                            interface_file=tfile.name,
                            **iface["build_interface"]
                        ),
                        iface["return"],
                    )

    # 'up' function tests: 1

    def test_up(self):
        """
        Test if it start up a network interface
        """
        self.assertEqual(debian_ip.down("eth0", "slave"), None)

        mock = MagicMock(return_value="Salt")
        with patch.dict(debian_ip.__salt__, {"cmd.run": mock}):
            self.assertEqual(debian_ip.up("eth0", "eth"), "Salt")

    # 'get_network_settings' function tests: 1

    def test_get_network_settings(self):
        """
        Test if it return the contents of the global network script.
        """
        with patch.dict(
            debian_ip.__grains__, {"osfullname": "Ubuntu", "osrelease": "14"}
        ), patch(
            "salt.modules.debian_ip._parse_hostname",
            MagicMock(return_value="SaltStack"),
        ), patch(
            "salt.modules.debian_ip._parse_domainname",
            MagicMock(return_value="saltstack.com"),
        ):
            mock_avai = MagicMock(return_value=True)
            with patch.dict(
                debian_ip.__salt__,
                {"service.available": mock_avai, "service.status": mock_avai},
            ):
                self.assertEqual(
                    debian_ip.get_network_settings(),
                    [
                        "NETWORKING=yes\n",
                        "HOSTNAME=SaltStack\n",
                        "DOMAIN=saltstack.com\n",
                    ],
                )

                mock = MagicMock(
                    side_effect=jinja2.exceptions.TemplateNotFound("error")
                )
                with patch.object(jinja2.Environment, "get_template", mock):
                    self.assertEqual(debian_ip.get_network_settings(), "")

    # 'get_routes' function tests: 1

    def test_get_routes(self):
        """
        Test if it return the routes for the interface
        """
        with patch("salt.modules.debian_ip._read_file", MagicMock(return_value="salt")):
            self.assertEqual(debian_ip.get_routes("eth0"), "saltsalt")

    # 'apply_network_settings' function tests: 1

    @pytest.mark.slow_test
    def test_apply_network_settings(self):
        """
        Test if it apply global network configuration.
        """
        mock = MagicMock(return_value=True)
        with patch.dict(
            debian_ip.__salt__,
            {"network.mod_hostname": mock, "service.stop": mock, "service.start": mock},
        ):
            self.assertEqual(debian_ip.apply_network_settings(), True)

    # 'build_network_settings' function tests: 1

    def test_build_network_settings(self):
        """
        Test if it build the global network script.
        """
        with patch(
            "salt.modules.debian_ip._parse_network_settings",
            MagicMock(
                return_value={
                    "networking": "yes",
                    "hostname": "Salt.saltstack.com",
                    "domainname": "saltstack.com",
                    "search": "test.saltstack.com",
                }
            ),
        ), patch(
            "salt.modules.debian_ip._write_file_network", MagicMock(return_value=True)
        ):
            with patch.dict(
                debian_ip.__grains__, {"osfullname": "Ubuntu", "osrelease": "14"}
            ):
                mock = MagicMock(return_value=True)
                with patch.dict(
                    debian_ip.__salt__,
                    {
                        "service.available": mock,
                        "service.disable": mock,
                        "service.enable": mock,
                    },
                ):
                    self.assertEqual(
                        debian_ip.build_network_settings(),
                        [
                            "NETWORKING=yes\n",
                            "HOSTNAME=Salt\n",
                            "DOMAIN=saltstack.com\n",
                            "SEARCH=test.saltstack.com\n",
                        ],
                    )

                    mock = MagicMock(
                        side_effect=jinja2.exceptions.TemplateNotFound("error")
                    )
                    with patch.object(jinja2.Environment, "get_template", mock):
                        self.assertEqual(debian_ip.build_network_settings(), "")

            with patch.dict(
                debian_ip.__grains__, {"osfullname": "Ubuntu", "osrelease": "10"}
            ):
                mock = MagicMock(return_value=True)
                with patch.dict(
                    debian_ip.__salt__,
                    {
                        "service.available": mock,
                        "service.disable": mock,
                        "service.enable": mock,
                    },
                ):
                    mock = MagicMock(
                        side_effect=jinja2.exceptions.TemplateNotFound("error")
                    )
                    with patch.object(jinja2.Environment, "get_template", mock):
                        self.assertEqual(debian_ip.build_network_settings(), "")

                    with patch.object(
                        debian_ip, "_read_temp", MagicMock(return_value=True)
                    ):
                        self.assertTrue(debian_ip.build_network_settings(test="True"))



import salt.modules.uptime as uptime
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import Mock
from tests.support.unit import TestCase


class RequestMock(Mock):
    """
    Request Mock
    """

    def get(self, *args, **kwargs):
        return RequestResponseMock()

    def put(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        return RequestPutResponseMock()

    def delete(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        return RequestResponseMock()


class RequestResponseMock(Mock):
    def json(self):
        return [
            {"url": "http://example.org", "_id": 1234},
        ]


class RequestPutResponseMock(Mock):

    ok = True

    def json(self):
        return {"_id": 4321}


REQUEST_MOCK = RequestMock()


class UptimeTestCase(TestCase, LoaderModuleMockMixin):
    """
    UptimeTestCase
    """

    def setup_loader_modules(self):
        return {
            uptime: {
                "__salt__": {
                    "pillar.get": Mock(return_value="http://localhost:5000"),
                    "requests.put": Mock(),
                },
                "requests": REQUEST_MOCK,
            }
        }

    def test_checks_list(self):
        ret = uptime.checks_list()
        self.assertListEqual(ret, ["http://example.org"])

    def test_checks_exists(self):
        self.assertTrue(uptime.check_exists("http://example.org") is True)

    def test_checks_create(self):
        self.assertRaises(CommandExecutionError, uptime.create, "http://example.org")
        self.assertEqual(4321, uptime.create("http://example.com"))
        self.assertEqual(("http://localhost:5000/api/checks",), REQUEST_MOCK.args)

    def test_checks_delete(self):
        self.assertRaises(CommandExecutionError, uptime.delete, "http://example.com")
        self.assertTrue(uptime.delete("http://example.org") is True)
        self.assertEqual(("http://localhost:5000/api/checks/1234",), REQUEST_MOCK.args)



"""
    tests.unit.utils.cache_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Test the salt cache objects
"""

import time

import pytest
import salt.config
import salt.loader
import salt.payload
import salt.utils.cache as cache
import salt.utils.data
import salt.utils.files


def test_sanity():
    """
    Make sure you can instantiate etc.
    """
    cd = cache.CacheDict(5)
    assert isinstance(cd, cache.CacheDict)

    # do some tests to make sure it looks like a dict
    assert "foo" not in cd
    cd["foo"] = "bar"
    assert cd["foo"] == "bar"
    del cd["foo"]
    assert "foo" not in cd


def test_ttl():
    cd = cache.CacheDict(0.1)
    cd["foo"] = "bar"
    assert "foo" in cd
    assert cd["foo"] == "bar"
    time.sleep(0.2)
    assert "foo" not in cd

    # make sure that a get would get a regular old key error
    with pytest.raises(KeyError):
        cd["foo"]  # pylint: disable=pointless-statement


@pytest.fixture
def cache_dir(tmp_path):
    cachedir = tmp_path / "cachedir"
    cachedir.mkdir()
    return cachedir


@pytest.fixture
def minion_config(cache_dir):
    opts = salt.config.DEFAULT_MINION_OPTS.copy()
    opts["cachedir"] = str(cache_dir)
    return opts


def test_smoke_context(minion_config):
    """
    Smoke test the context cache
    """
    context_cache = cache.ContextCache(minion_config, "cache_test")

    data = {"a": "b"}
    context_cache.cache_context(data.copy())

    ret = context_cache.get_cache_context()

    assert ret == data


@pytest.fixture
def cache_mod_name():
    return "cache_mod"


@pytest.fixture
def cache_mods_path(tmp_path, cache_mod_name):
    _cache_mods_path = tmp_path / "cache_mods"
    mod_contents = """
    import salt.utils.cache

    def __virtual__():
        return True

    @salt.utils.cache.context_cache
    def test_context_module():
        if "called" in __context__:
            __context__["called"] += 1
        else:
            __context__["called"] = 0
        return __context__.value()

    @salt.utils.cache.context_cache
    def test_compare_context():
        return __context__.value()
    """
    with pytest.helpers.temp_file(
        cache_mod_name + ".py", mod_contents, _cache_mods_path
    ):
        yield _cache_mods_path


def test_context_wrapper(minion_config, cache_mods_path):
    """
    Test to ensure that a module which decorates itself
    with a context cache can store and retrieve its contextual
    data
    """

    loader = salt.loader.LazyLoader(
        [str(cache_mods_path)],
        tag="rawmodule",
        virtual_enable=False,
        opts=minion_config,
    )

    cache_test_func = loader["cache_mod.test_context_module"]

    assert cache_test_func()["called"] == 0
    assert cache_test_func()["called"] == 1


def test_set_cache(minion_config, cache_mods_path, cache_mod_name, cache_dir):
    """
    Tests to ensure the cache is written correctly
    """

    context = {"c": "d"}
    loader = salt.loader.LazyLoader(
        [str(cache_mods_path)],
        tag="rawmodule",
        virtual_enable=False,
        opts=minion_config,
        pack={"__context__": context, "__opts__": minion_config},
    )

    cache_test_func = loader["cache_mod.test_context_module"]

    # Call the function to trigger the context cache
    assert cache_test_func()["called"] == 0
    assert cache_test_func()["called"] == 1
    assert cache_test_func()["called"] == 2

    cache_file_name = "salt.loaded.ext.rawmodule.{}.p".format(cache_mod_name)

    cached_file = cache_dir / "context" / cache_file_name
    assert cached_file.exists()

    # Test manual de-serialize
    target_cache_data = salt.utils.data.decode(
        salt.payload.loads(cached_file.read_bytes())
    )
    assert target_cache_data == dict(context, called=1)

    # Test cache de-serialize
    cc = cache.ContextCache(
        minion_config, "salt.loaded.ext.rawmodule.{}".format(cache_mod_name)
    )
    retrieved_cache = cc.get_cache_context()
    assert retrieved_cache == dict(context, called=1)


def test_refill_cache(minion_config, cache_mods_path):
    """
    Tests to ensure that the context cache can rehydrate a wrapped function
    """
    context = {"c": "d"}
    loader = salt.loader.LazyLoader(
        [str(cache_mods_path)],
        tag="rawmodule",
        virtual_enable=False,
        opts=minion_config,
        pack={"__context__": context, "__opts__": minion_config},
    )

    cache_test_func = loader["cache_mod.test_compare_context"]
    # First populate the cache
    ret = cache_test_func()
    assert ret == context

    # Then try to rehydrate a func
    context_copy = context.copy()
    context.clear()

    # Compare to the context before it was emptied
    ret = cache_test_func()
    assert ret == context_copy


def test_everything(tmp_path):
    """
    Make sure you can instantiate, add, update, remove, expire
    """
    path = str(tmp_path / "cachedir")

    # test instantiation
    cd = cache.CacheDisk(0.3, path)
    assert isinstance(cd, cache.CacheDisk)

    # test to make sure it looks like a dict
    assert "foo" not in cd
    cd["foo"] = "bar"
    assert "foo" in cd
    assert cd["foo"] == "bar"
    del cd["foo"]
    assert "foo" not in cd

    # test persistence
    cd["foo"] = "bar"
    cd2 = cache.CacheDisk(0.3, path)
    assert "foo" in cd2
    assert cd2["foo"] == "bar"

    # test ttl
    time.sleep(0.5)
    assert "foo" not in cd
    assert "foo" not in cd2



"""
Test AnsibleGate State Module
"""

import shutil

import pytest
import salt.utils.files
import salt.utils.path
import yaml
from saltfactories.utils.functional import StateResult
from tests.support.runtests import RUNTIME_VARS

pytestmark = [
    # root access is necessary because the playbooks install/uninstall software
    pytest.mark.skip_if_not_root,
    # Because of the above, these are also destructive tests
    pytest.mark.destructive_test,
    pytest.mark.skip_if_binaries_missing(
        "ansible-playbook", reason="ansible-playbook is not installed"
    ),
]


@pytest.fixture(scope="module")
def ansible_inventory_directory(tmp_path_factory, grains):
    if grains["os_family"] != "RedHat":
        pytest.skip("Currently, the test targets the RedHat OS familly only.")
    tmp_dir = tmp_path_factory.mktemp("ansible")
    try:
        yield tmp_dir
    finally:
        shutil.rmtree(str(tmp_dir))


@pytest.fixture(scope="module", autouse=True)
def ansible_inventory(ansible_inventory_directory, sshd_server):
    inventory = str(ansible_inventory_directory / "inventory")
    client_key = str(sshd_server.config_dir / "client_key")
    data = {
        "all": {
            "hosts": {
                "localhost": {
                    "ansible_host": "127.0.0.1",
                    "ansible_port": sshd_server.listen_port,
                    "ansible_user": RUNTIME_VARS.RUNNING_TESTS_USER,
                    "ansible_ssh_private_key_file": client_key,
                    "ansible_ssh_extra_args": (
                        "-o StrictHostKeyChecking=false "
                        "-o UserKnownHostsFile=/dev/null "
                    ),
                },
            },
        },
    }
    with salt.utils.files.fopen(inventory, "w") as yaml_file:
        yaml.dump(data, yaml_file, default_flow_style=False)
    return inventory


@pytest.mark.requires_sshd_server
def test_ansible_playbook(salt_call_cli, ansible_inventory):
    ret = salt_call_cli.run(
        "state.single",
        "ansible.playbooks",
        name="remove.yml",
        git_repo="https://github.com/saltstack/salt-test-suite-ansible-playbooks.git",
        ansible_kwargs={"inventory": ansible_inventory},
    )
    assert ret.exitcode == 0
    assert StateResult(ret.json).result is True

    ret = salt_call_cli.run(
        "state.single",
        "ansible.playbooks",
        name="install.yml",
        git_repo="https://github.com/saltstack/salt-test-suite-ansible-playbooks.git",
        ansible_kwargs={"inventory": ansible_inventory},
    )
    assert ret.exitcode == 0
    assert StateResult(ret.json).result is True



"""
Management of zc.buildout
=========================

This module is inspired from minitage's buildout maker
(https://github.com/minitage/minitage/blob/master/src/minitage/core/makers/buildout.py)

.. versionadded:: 2016.3.0

.. note::

    This state module is beta; the API is subject to change and no promise
    as to performance or functionality is yet present

Available Functions
-------------------

- built

  .. code-block:: yaml

      installed1
        buildout.installed:
          - name: /path/to/buildout

      installed2
        buildout.installed:
          - name: /path/to/buildout
          - parts:
            - a
            - b
          - python: /path/to/pythonpath/bin/python
          - unless: /bin/test_something_installed
          - onlyif: /bin/test_else_installed

"""

import logging
import sys

# Define the module's virtual name
__virtualname__ = "buildout"
log = logging.getLogger(__name__)


def __virtual__():
    """
    Only load if zc.buildout libs available
    """
    if "buildout.buildout" in __salt__:
        return __virtualname__
    return (False, "buildout module could not be loaded")


INVALID_RESPONSE = "Unexpected response from docker"
VALID_RESPONSE = ""
NOTSET = object()
MAPPING_CACHE = {}
FN_CACHE = {}


def __salt(fn):
    if fn not in FN_CACHE:
        FN_CACHE[fn] = __salt__[fn]
    return FN_CACHE[fn]


def _ret_status(
    exec_status=None, name="", comment="", result=None, quiet=False, changes=None
):
    if not changes:
        changes = {}
    if exec_status is None:
        exec_status = {}
    if exec_status:
        if result is None:
            result = exec_status["status"]
        scomment = exec_status.get("comment", None)
        if scomment:
            comment += "\n" + scomment
        out = exec_status.get("out", "")
        if not quiet:
            if out:
                if isinstance(out, str):
                    comment += "\n" + out
            outlog = exec_status.get("outlog", None)
            if outlog:
                if isinstance(outlog, str):
                    comment += "\n" + outlog
    return {
        "changes": changes,
        "result": result,
        "name": name,
        "comment": comment,
    }


def _valid(exec_status=None, name="", comment="", changes=None):
    return _ret_status(
        exec_status=exec_status,
        comment=comment,
        name=name,
        changes=changes,
        result=True,
    )


def _invalid(exec_status=None, name="", comment="", changes=None):
    return _ret_status(
        exec_status=exec_status,
        comment=comment,
        name=name,
        changes=changes,
        result=False,
    )


def installed(
    name,
    config="buildout.cfg",
    quiet=False,
    parts=None,
    user=None,
    env=(),
    buildout_ver=None,
    test_release=False,
    distribute=None,
    new_st=None,
    offline=False,
    newest=False,
    python=sys.executable,
    debug=False,
    verbose=False,
    unless=None,
    onlyif=None,
    use_vt=False,
    loglevel="debug",
    **kwargs
):
    """
    Install buildout in a specific directory

    It is a thin wrapper to modules.buildout.buildout

    name
        directory to execute in

    quiet

        do not output console & logs

    config
        buildout config to use (default: buildout.cfg)

    parts
        specific buildout parts to run

    user
        user used to run buildout as

        .. versionadded:: 2014.1.4

    env
        environment variables to set when running

    buildout_ver
        force a specific buildout version (1 | 2)

    test_release
        buildout accept test release

    new_st
        Forcing use of setuptools >= 0.7

    distribute
        use distribute over setuptools if possible

    offline
        does buildout run offline

    python
        python to use

    debug
        run buildout with -D debug flag

    onlyif
        Only execute cmd if statement on the host return 0

    unless
        Do not execute cmd if statement on the host return 0

    newest
        run buildout in newest mode

    verbose
        run buildout in verbose mode (-vvvvv)

    use_vt
        Use the new salt VT to stream output [experimental]

    loglevel
        loglevel for buildout commands
    """
    ret = {}

    if "group" in kwargs:
        log.warning("Passing 'group' is deprecated, just remove it")
    output_loglevel = kwargs.get("output_loglevel", None)
    if output_loglevel and not loglevel:
        log.warning(
            "Passing 'output_loglevel' is deprecated, please use loglevel instead"
        )
    try:
        test_release = int(test_release)
    except ValueError:
        test_release = None

    func = __salt("buildout.buildout")
    kwargs = dict(
        directory=name,
        config=config,
        parts=parts,
        runas=user,
        env=env,
        buildout_ver=buildout_ver,
        test_release=test_release,
        distribute=distribute,
        new_st=new_st,
        offline=offline,
        newest=newest,
        python=python,
        debug=debug,
        verbose=verbose,
        onlyif=onlyif,
        unless=unless,
        use_vt=use_vt,
        loglevel=loglevel,
    )
    ret.update(_ret_status(func(**kwargs), name, quiet=quiet))
    return ret



"""
Functions for working with the codepage on Windows systems
"""

import logging
from contextlib import contextmanager

from salt.exceptions import CodePageError

log = logging.getLogger(__name__)

try:
    import pywintypes
    import win32console

    HAS_WIN32 = True
except ImportError:
    HAS_WIN32 = False


# Although utils are often directly imported, it is also possible to use the loader.
def __virtual__():
    """
    Only load if Win32 Libraries are installed
    """
    if not HAS_WIN32:
        return False, "This utility requires pywin32"

    return "win_chcp"


@contextmanager
def chcp(page_id, raise_error=False):
    """
    Gets or sets the codepage of the shell.

    Args:

        page_id (str, int):
            A number representing the codepage.

        raise_error (bool):
            ``True`` will raise an error if the codepage fails to change.
            ``False`` will suppress the error

    Returns:
        int: A number representing the codepage

    Raises:
        CodePageError: On unsuccessful codepage change
    """
    if not isinstance(page_id, int):
        try:
            page_id = int(page_id)
        except ValueError:
            error = "The `page_id` needs to be an integer, not {}".format(type(page_id))
            if raise_error:
                raise CodePageError(error)
            log.error(error)
            return -1

    previous_page_id = get_codepage_id(raise_error=raise_error)

    if page_id and previous_page_id and page_id != previous_page_id:
        set_code_page = True
    else:
        set_code_page = False

    try:
        if set_code_page:
            set_codepage_id(page_id, raise_error=raise_error)

        # Subprocesses started from now will use the set code page id
        yield
    finally:
        if set_code_page:
            # Reset to the old code page
            set_codepage_id(previous_page_id, raise_error=raise_error)


def get_codepage_id(raise_error=False):
    """
    Get the currently set code page on windows

    Args:

        raise_error (bool):
            ``True`` will raise an error if the codepage fails to change.
            ``False`` will suppress the error

    Returns:
        int: A number representing the codepage

    Raises:
        CodePageError: On unsuccessful codepage change
    """
    try:
        return win32console.GetConsoleCP()
    except pywintypes.error as exc:
        _, _, msg = exc.args
        error = "Failed to get the windows code page: {}".format(msg)
        if raise_error:
            raise CodePageError(error)
        else:
            log.error(error)
        return -1


def set_codepage_id(page_id, raise_error=False):
    """
    Set the code page on windows

    Args:

        page_id (str, int):
            A number representing the codepage.

        raise_error (bool):
            ``True`` will raise an error if the codepage fails to change.
            ``False`` will suppress the error

    Returns:
        int: A number representing the codepage

    Raises:
        CodePageError: On unsuccessful codepage change
    """
    if not isinstance(page_id, int):
        try:
            page_id = int(page_id)
        except ValueError:
            error = "The `page_id` needs to be an integer, not {}".format(type(page_id))
            if raise_error:
                raise CodePageError(error)
            log.error(error)
            return -1
    try:
        win32console.SetConsoleCP(page_id)
        return get_codepage_id(raise_error=raise_error)
    except pywintypes.error as exc:
        _, _, msg = exc.args
        error = "Failed to set the windows code page: {}".format(msg)
        if raise_error:
            raise CodePageError(error)
        else:
            log.error(error)
        return -1



import salt.pillar.sqlcipher as sqlcipher
from tests.support.unit import TestCase


class SQLCipherPillarTestCase(TestCase):
    maxDiff = None

    def test_001_extract_queries_list(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        args, kwargs = (
            [
                "SELECT blah",
                "SELECT blah2",
                ("SELECT blah3",),
                ("SELECT blah4", 2),
                {"query": "SELECT blah5"},
                {"query": "SELECT blah6", "depth": 2},
                {"query": "SELECT blah7", "as_list": True},
                {"query": "SELECT blah8", "with_lists": "1"},
                {"query": "SELECT blah9", "with_lists": "1,2"},
            ],
            {},
        )
        qbuffer = return_data.extract_queries(args, kwargs)
        self.assertEqual(
            [
                [
                    None,
                    {
                        "query": "SELECT blah",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah2",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah3",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah4",
                        "depth": 2,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah5",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah6",
                        "depth": 2,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah7",
                        "depth": 0,
                        "as_list": True,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah8",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": [1],
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah9",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": [1, 2],
                        "ignore_null": False,
                    },
                ],
            ],
            qbuffer,
        )

    def test_002_extract_queries_kwarg(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        args, kwargs = (
            [],
            {
                "1": "SELECT blah",
                "2": "SELECT blah2",
                "3": ("SELECT blah3",),
                "4": ("SELECT blah4", 2),
                "5": {"query": "SELECT blah5"},
                "6": {"query": "SELECT blah6", "depth": 2},
                "7": {"query": "SELECT blah7", "as_list": True},
            },
        )
        qbuffer = return_data.extract_queries(args, kwargs)
        self.assertEqual(
            [
                [
                    "1",
                    {
                        "query": "SELECT blah",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "2",
                    {
                        "query": "SELECT blah2",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "3",
                    {
                        "query": "SELECT blah3",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "4",
                    {
                        "query": "SELECT blah4",
                        "depth": 2,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "5",
                    {
                        "query": "SELECT blah5",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "6",
                    {
                        "query": "SELECT blah6",
                        "depth": 2,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "7",
                    {
                        "query": "SELECT blah7",
                        "depth": 0,
                        "as_list": True,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
            ],
            qbuffer,
        )

    def test_003_extract_queries_mixed(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        args, kwargs = (
            [
                "SELECT blah1",
                ("SELECT blah2", 2),
                {"query": "SELECT blah3", "as_list": True},
            ],
            {
                "1": "SELECT blah1",
                "2": ("SELECT blah2", 2),
                "3": {"query": "SELECT blah3", "as_list": True},
            },
        )
        qbuffer = return_data.extract_queries(args, kwargs)
        self.assertEqual(
            [
                [
                    None,
                    {
                        "query": "SELECT blah1",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah2",
                        "depth": 2,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah3",
                        "depth": 0,
                        "as_list": True,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "1",
                    {
                        "query": "SELECT blah1",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "2",
                    {
                        "query": "SELECT blah2",
                        "depth": 2,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "3",
                    {
                        "query": "SELECT blah3",
                        "depth": 0,
                        "as_list": True,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
            ],
            qbuffer,
        )

    def test_004_extract_queries_bogus_list(self):
        # This test is specifically checking that empty queries are dropped
        return_data = sqlcipher.SQLCipherExtPillar()
        args, kwargs = (
            [
                "SELECT blah",
                "",
                "SELECT blah2",
                ("SELECT blah3",),
                ("",),
                ("SELECT blah4", 2),
                tuple(),
                ("SELECT blah5",),
                {"query": "SELECT blah6"},
                {"query": ""},
                {"query": "SELECT blah7", "depth": 2},
                {"not_a_query": "in sight!"},
                {"query": "SELECT blah8", "as_list": True},
            ],
            {},
        )
        qbuffer = return_data.extract_queries(args, kwargs)
        self.assertEqual(
            [
                [
                    None,
                    {
                        "query": "SELECT blah",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah2",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah3",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah4",
                        "depth": 2,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah5",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah6",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah7",
                        "depth": 2,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    None,
                    {
                        "query": "SELECT blah8",
                        "depth": 0,
                        "as_list": True,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
            ],
            qbuffer,
        )

    def test_005_extract_queries_bogus_kwargs(self):
        # this test is cut down as most of the path matches test_*_bogus_list
        return_data = sqlcipher.SQLCipherExtPillar()
        args, kwargs = [], {"1": "SELECT blah", "2": "", "3": "SELECT blah2"}
        qbuffer = return_data.extract_queries(args, kwargs)
        self.assertEqual(
            [
                [
                    "1",
                    {
                        "query": "SELECT blah",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
                [
                    "3",
                    {
                        "query": "SELECT blah2",
                        "depth": 0,
                        "as_list": False,
                        "with_lists": None,
                        "ignore_null": False,
                    },
                ],
            ],
            qbuffer,
        )

    def test_011_enter_root(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.enter_root("test")
        self.assertEqual(return_data.result["test"], return_data.focus)
        return_data.enter_root(None)
        self.assertEqual(return_data.result, return_data.focus)

    def test_021_process_fields(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b"], 0)
        self.assertEqual(return_data.num_fields, 2)
        self.assertEqual(return_data.depth, 1)
        return_data.process_fields(["a", "b"], 2)
        self.assertEqual(return_data.num_fields, 2)
        self.assertEqual(return_data.depth, 1)
        return_data.process_fields(["a", "b", "c", "d"], 0)
        self.assertEqual(return_data.num_fields, 4)
        self.assertEqual(return_data.depth, 3)
        return_data.process_fields(["a", "b", "c", "d"], 1)
        self.assertEqual(return_data.num_fields, 4)
        self.assertEqual(return_data.depth, 1)
        return_data.process_fields(["a", "b", "c", "d"], 2)
        self.assertEqual(return_data.num_fields, 4)
        self.assertEqual(return_data.depth, 2)
        return_data.process_fields(["a", "b", "c", "d"], 3)
        self.assertEqual(return_data.num_fields, 4)
        self.assertEqual(return_data.depth, 3)
        return_data.process_fields(["a", "b", "c", "d"], 4)
        self.assertEqual(return_data.num_fields, 4)
        self.assertEqual(return_data.depth, 3)

    def test_111_process_results_legacy(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b"], 0)
        return_data.with_lists = []
        return_data.process_results([[1, 2]])
        self.assertEqual({1: 2}, return_data.result)

    def test_112_process_results_legacy_multiple(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b"], 0)
        return_data.with_lists = []
        return_data.process_results([[1, 2], [3, 4], [5, 6]])
        self.assertEqual({1: 2, 3: 4, 5: 6}, return_data.result)

    def test_121_process_results_depth_0(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [5, 6, 7, 8]])
        self.assertEqual({1: {2: {3: 4}}, 5: {6: {7: 8}}}, return_data.result)

    def test_122_process_results_depth_1(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 1)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [5, 6, 7, 8]])
        self.assertEqual(
            {1: {"b": 2, "c": 3, "d": 4}, 5: {"b": 6, "c": 7, "d": 8}},
            return_data.result,
        )

    def test_123_process_results_depth_2(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 2)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [5, 6, 7, 8]])
        self.assertEqual(
            {1: {2: {"c": 3, "d": 4}}, 5: {6: {"c": 7, "d": 8}}}, return_data.result
        )

    def test_124_process_results_depth_3(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 3)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [5, 6, 7, 8]])
        self.assertEqual({1: {2: {3: 4}}, 5: {6: {7: 8}}}, return_data.result)

    def test_125_process_results_depth_4(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 4)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [5, 6, 7, 8]])
        self.assertEqual({1: {2: {3: 4}}, 5: {6: {7: 8}}}, return_data.result)

    def test_131_process_results_overwrite_legacy_multiple(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b"], 0)
        return_data.with_lists = []
        return_data.process_results([[1, 2], [3, 4], [1, 6]])
        self.assertEqual({1: 6, 3: 4}, return_data.result)

    def test_132_process_results_merge_depth_0(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [1, 6, 7, 8]])
        self.assertEqual({1: {2: {3: 4}, 6: {7: 8}}}, return_data.result)

    def test_133_process_results_overwrite_depth_0(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [1, 2, 3, 8]])
        self.assertEqual({1: {2: {3: 8}}}, return_data.result)

    def test_134_process_results_deepmerge_depth_0(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [1, 2, 7, 8]])
        self.assertEqual({1: {2: {3: 4, 7: 8}}}, return_data.result)

    def test_135_process_results_overwrite_depth_1(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 1)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [1, 6, 7, 8]])
        self.assertEqual({1: {"b": 6, "c": 7, "d": 8}}, return_data.result)

    def test_136_process_results_merge_depth_2(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 2)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [1, 6, 7, 8]])
        self.assertEqual(
            {1: {2: {"c": 3, "d": 4}, 6: {"c": 7, "d": 8}}}, return_data.result
        )

    def test_137_process_results_overwrite_depth_2(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 2)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4], [1, 2, 7, 8]])
        self.assertEqual({1: {2: {"c": 7, "d": 8}}}, return_data.result)

    def test_201_process_results_complexity_multiresults(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 2)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_results([[1, 2, 7, 8]])
        self.assertEqual({1: {2: {"c": 7, "d": 8}}}, return_data.result)

    def test_202_process_results_complexity_as_list(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 2)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.as_list = True
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_results([[1, 2, 7, 8]])
        self.assertEqual({1: {2: {"c": [3, 7], "d": [4, 8]}}}, return_data.result)

    def test_203_process_results_complexity_as_list_deeper(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.as_list = True
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_results([[1, 2, 3, 8]])
        self.assertEqual({1: {2: {3: [4, 8]}}}, return_data.result)

    def test_204_process_results_complexity_as_list_mismatch_depth(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = True
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_results([[1, 2, 3, 5]])
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 6, 7]])
        self.assertEqual({1: {2: {3: [4, 5, {6: 7}]}}}, return_data.result)

    def test_205_process_results_complexity_as_list_mismatch_depth_reversed(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = True
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 6, 7]])
        return_data.process_results([[1, 2, 3, 8, 9]])
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_results([[1, 2, 3, 5]])
        self.assertEqual({1: {2: {3: [{6: 7, 8: 9}, 4, 5]}}}, return_data.result)

    def test_206_process_results_complexity_as_list_mismatch_depth_weird_order(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = True
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 6, 7]])
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 8, 9]])
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 5]])
        self.assertEqual({1: {2: {3: [{6: 7}, 4, {8: 9}, 5]}}}, return_data.result)

    def test_207_process_results_complexity_collision_mismatch_depth(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = False
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_results([[1, 2, 3, 5]])
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 6, 7]])
        self.assertEqual({1: {2: {3: {6: 7}}}}, return_data.result)

    def test_208_process_results_complexity_collision_mismatch_depth_reversed(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = False
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 6, 7]])
        return_data.process_results([[1, 2, 3, 8, 9]])
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_results([[1, 2, 3, 5]])
        self.assertEqual({1: {2: {3: 5}}}, return_data.result)

    def test_209_process_results_complexity_collision_mismatch_depth_weird_order(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = False
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 6, 7]])
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 8, 9]])
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 5]])
        self.assertEqual({1: {2: {3: 5}}}, return_data.result)

    def test_20A_process_results_complexity_as_list_vary(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = True
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d", "e"], 0)
        return_data.process_results([[1, 2, 3, 6, 7]])
        return_data.process_results([[1, 2, 3, 8, 9]])
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.as_list = False
        return_data.process_results([[1, 2, 3, 5]])
        self.assertEqual({1: {2: {3: 5}}}, return_data.result)

    def test_207_process_results_complexity_roots_collision(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = False
        return_data.with_lists = []
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d"], 0)
        return_data.process_results([[1, 2, 3, 4]])
        return_data.enter_root(1)
        return_data.process_results([[5, 6, 7, 8]])
        self.assertEqual({1: {5: {6: {7: 8}}}}, return_data.result)

    def test_301_process_results_with_lists(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = False
        return_data.with_lists = [1, 3]
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d", "e", "v"], 0)
        return_data.process_results(
            [
                ["a", "b", "c", "d", "e", 1],
                ["a", "b", "c", "f", "g", 2],
                ["a", "z", "h", "y", "j", 3],
                ["a", "z", "h", "y", "k", 4],
            ]
        )
        self.assertEqual(
            sorted({"a": [{"c": [{"e": 1}, {"g": 2}]}, {"h": [{"j": 3, "k": 4}]}]}),
            sorted(return_data.result),
        )

    def test_302_process_results_with_lists_consecutive(self):
        return_data = sqlcipher.SQLCipherExtPillar()
        return_data.as_list = False
        return_data.with_lists = [1, 2, 3]
        return_data.enter_root(None)
        return_data.process_fields(["a", "b", "c", "d", "e", "v"], 0)
        return_data.process_results(
            [
                ["a", "b", "c", "d", "e", 1],
                ["a", "b", "c", "f", "g", 2],
                ["a", "z", "h", "y", "j", 3],
                ["a", "z", "h", "y", "k", 4],
            ]
        )
        self.assertEqual(
            sorted({"a": [[[{"e": 1}, {"g": 2}]], [[{"j": 3, "k": 4}]]]}),
            sorted(return_data.result),
        )



"""
Apache Libcloud Load Balancer Management
========================================

Connection module for Apache Libcloud Storage load balancer management for a full list
of supported clouds, see http://libcloud.readthedocs.io/en/latest/loadbalancer/supported_providers.html

Clouds include Amazon ELB, ALB, Google, Aliyun, CloudStack, Softlayer

.. versionadded:: 2018.3.0

:configuration:
    This module uses a configuration profile for one or multiple Storage providers

    .. code-block:: yaml

        libcloud_loadbalancer:
            profile_test1:
              driver: gce
              key: GOOG0123456789ABCXYZ
              secret: mysecret
            profile_test2:
              driver: alb
              key: 12345
              secret: mysecret

:depends: apache-libcloud
"""
# keep lint from choking on _get_conn and _cache_id
# pylint: disable=E0602


import logging

import salt.utils.args
import salt.utils.compat
from salt.utils.versions import LooseVersion as _LooseVersion

log = logging.getLogger(__name__)

REQUIRED_LIBCLOUD_VERSION = "1.5.0"
try:
    # pylint: disable=unused-import
    import libcloud
    from libcloud.loadbalancer.providers import get_driver
    from libcloud.loadbalancer.base import Member, Algorithm

    # pylint: enable=unused-import
    if hasattr(libcloud, "__version__") and _LooseVersion(
        libcloud.__version__
    ) < _LooseVersion(REQUIRED_LIBCLOUD_VERSION):
        raise ImportError()
    logging.getLogger("libcloud").setLevel(logging.CRITICAL)
    HAS_LIBCLOUD = True
except ImportError:
    HAS_LIBCLOUD = False


def __virtual__():
    """
    Only load if libcloud libraries exist.
    """
    if not HAS_LIBCLOUD:
        return (
            False,
            "A apache-libcloud library with version at least {} was not found".format(
                REQUIRED_LIBCLOUD_VERSION
            ),
        )
    return True


def _algorithm_maps():
    return {
        "RANDOM": Algorithm.RANDOM,
        "ROUND_ROBIN": Algorithm.ROUND_ROBIN,
        "LEAST_CONNECTIONS": Algorithm.LEAST_CONNECTIONS,
        "WEIGHTED_ROUND_ROBIN": Algorithm.WEIGHTED_ROUND_ROBIN,
        "WEIGHTED_LEAST_CONNECTIONS": Algorithm.WEIGHTED_LEAST_CONNECTIONS,
        "SHORTEST_RESPONSE": Algorithm.SHORTEST_RESPONSE,
        "PERSISTENT_IP": Algorithm.PERSISTENT_IP,
    }


def _get_driver(profile):
    config = __salt__["config.option"]("libcloud_loadbalancer")[profile]
    cls = get_driver(config["driver"])
    args = config.copy()
    del args["driver"]
    args["key"] = config.get("key")
    args["secret"] = config.get("secret", None)
    if args["secret"] is None:
        del args["secret"]
    args["secure"] = config.get("secure", True)
    args["host"] = config.get("host", None)
    args["port"] = config.get("port", None)
    return cls(**args)


def list_balancers(profile, **libcloud_kwargs):
    """
    Return a list of load balancers.

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's list_balancers method
    :type  libcloud_kwargs: ``dict``

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.list_balancers profile1
    """
    conn = _get_driver(profile=profile)
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    balancers = conn.list_balancers(**libcloud_kwargs)
    ret = []
    for balancer in balancers:
        ret.append(_simple_balancer(balancer))
    return ret


def list_protocols(profile, **libcloud_kwargs):
    """
    Return a list of supported protocols.

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's list_protocols method
    :type  libcloud_kwargs: ``dict``

    :return: a list of supported protocols
    :rtype: ``list`` of ``str``

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.list_protocols profile1
    """
    conn = _get_driver(profile=profile)
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    return conn.list_protocols(**libcloud_kwargs)


def create_balancer(
    name, port, protocol, profile, algorithm=None, members=None, **libcloud_kwargs
):
    """
    Create a new load balancer instance

    :param name: Name of the new load balancer (required)
    :type  name: ``str``

    :param port: Port the load balancer should listen on, defaults to 80
    :type  port: ``str``

    :param protocol: Loadbalancer protocol, defaults to http.
    :type  protocol: ``str``

    :param algorithm: Load balancing algorithm, defaults to ROUND_ROBIN. See Algorithm type
        in Libcloud documentation for a full listing.
    :type algorithm: ``str``

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's create_balancer method
    :type  libcloud_kwargs: ``dict``

    :return: The details of the new balancer

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.create_balancer my_balancer 80 http profile1
    """
    if algorithm is None:
        algorithm = Algorithm.ROUND_ROBIN
    else:
        if isinstance(algorithm, str):
            algorithm = _algorithm_maps()[algorithm]
    starting_members = []
    if members is not None:
        if isinstance(members, list):
            for m in members:
                starting_members.append(Member(id=None, ip=m["ip"], port=m["port"]))
        else:
            raise ValueError("members must be of type list")

    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    conn = _get_driver(profile=profile)
    balancer = conn.create_balancer(
        name, port, protocol, algorithm, starting_members, **libcloud_kwargs
    )
    return _simple_balancer(balancer)


def destroy_balancer(balancer_id, profile, **libcloud_kwargs):
    """
    Destroy a load balancer

    :param balancer_id: LoadBalancer ID which should be used
    :type  balancer_id: ``str``

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's destroy_balancer method
    :type  libcloud_kwargs: ``dict``

    :return: ``True`` if the destroy was successful, otherwise ``False``.
    :rtype: ``bool``

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.destroy_balancer balancer_1 profile1
    """
    conn = _get_driver(profile=profile)
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    balancer = conn.get_balancer(balancer_id)
    return conn.destroy_balancer(balancer, **libcloud_kwargs)


def get_balancer_by_name(name, profile, **libcloud_kwargs):
    """
    Get the details for a load balancer by name

    :param name: Name of a load balancer you want to fetch
    :type  name: ``str``

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's list_balancers method
    :type  libcloud_kwargs: ``dict``

    :return: the load balancer details

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.get_balancer_by_name my_balancer profile1
    """
    conn = _get_driver(profile=profile)
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    balancers = conn.list_balancers(**libcloud_kwargs)
    match = [b for b in balancers if b.name == name]
    if len(match) == 1:
        return _simple_balancer(match[0])
    elif len(match) > 1:
        raise ValueError("Ambiguous argument, found mulitple records")
    else:
        raise ValueError("Bad argument, found no records")


def get_balancer(balancer_id, profile, **libcloud_kwargs):
    """
    Get the details for a load balancer by ID

    :param balancer_id: id of a load balancer you want to fetch
    :type  balancer_id: ``str``

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's get_balancer method
    :type  libcloud_kwargs: ``dict``

    :return: the load balancer details

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.get_balancer balancer123 profile1
    """
    conn = _get_driver(profile=profile)
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    balancer = conn.get_balancer(balancer_id, **libcloud_kwargs)
    return _simple_balancer(balancer)


def list_supported_algorithms(profile, **libcloud_kwargs):
    """
    Get the supported algorithms for a profile

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's list_supported_algorithms method
    :type  libcloud_kwargs: ``dict``

    :return: The supported algorithms

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.list_supported_algorithms profile1
    """
    conn = _get_driver(profile=profile)
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    return conn.list_supported_algorithms(**libcloud_kwargs)


def balancer_attach_member(
    balancer_id, ip, port, profile, extra=None, **libcloud_kwargs
):
    """
    Add a new member to the load balancer

    :param balancer_id: id of a load balancer you want to fetch
    :type  balancer_id: ``str``

    :param ip: IP address for the new member
    :type  ip: ``str``

    :param port: Port for the new member
    :type  port: ``int``

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's balancer_attach_member method
    :type  libcloud_kwargs: ``dict``

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.balancer_attach_member balancer123 1.2.3.4 80 profile1
    """
    conn = _get_driver(profile=profile)
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    member = Member(id=None, ip=ip, port=port, balancer=None, extra=extra)
    balancer = conn.get_balancer(balancer_id)
    member_saved = conn.balancer_attach_member(balancer, member, **libcloud_kwargs)
    return _simple_member(member_saved)


def balancer_detach_member(balancer_id, member_id, profile, **libcloud_kwargs):
    """
    Add a new member to the load balancer

    :param balancer_id: id of a load balancer you want to fetch
    :type  balancer_id: ``str``

    :param ip: IP address for the new member
    :type  ip: ``str``

    :param port: Port for the new member
    :type  port: ``int``

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's balancer_detach_member method
    :type  libcloud_kwargs: ``dict``

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.balancer_detach_member balancer123 member123 profile1
    """
    conn = _get_driver(profile=profile)
    balancer = conn.get_balancer(balancer_id)
    members = conn.balancer_list_members(balancer=balancer)
    match = [member for member in members if member.id == member_id]
    if len(match) > 1:
        raise ValueError("Ambiguous argument, found mulitple records")
    elif not match:
        raise ValueError("Bad argument, found no records")
    else:
        member = match[0]
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    return conn.balancer_detach_member(
        balancer=balancer, member=member, **libcloud_kwargs
    )


def list_balancer_members(balancer_id, profile, **libcloud_kwargs):
    """
    List the members of a load balancer

    :param balancer_id: id of a load balancer you want to fetch
    :type  balancer_id: ``str``

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's list_balancer_members method
    :type  libcloud_kwargs: ``dict``

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_storage.list_balancer_members balancer123 profile1
    """
    conn = _get_driver(profile=profile)
    balancer = conn.get_balancer(balancer_id)
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    members = conn.balancer_list_members(balancer=balancer, **libcloud_kwargs)
    return [_simple_member(member) for member in members]


def extra(method, profile, **libcloud_kwargs):
    """
    Call an extended method on the driver

    :param method: Driver's method name
    :type  method: ``str``

    :param profile: The profile key
    :type  profile: ``str``

    :param libcloud_kwargs: Extra arguments for the driver's method
    :type  libcloud_kwargs: ``dict``

    CLI Example:

    .. code-block:: bash

        salt myminion libcloud_loadbalancer.extra ex_get_permissions google container_name=my_container object_name=me.jpg --out=yaml
    """
    libcloud_kwargs = salt.utils.args.clean_kwargs(**libcloud_kwargs)
    conn = _get_driver(profile=profile)
    connection_method = getattr(conn, method)
    return connection_method(**libcloud_kwargs)


def _simple_balancer(balancer):
    return {
        "id": balancer.id,
        "name": balancer.name,
        "state": balancer.state,
        "ip": balancer.ip,
        "port": balancer.port,
        "extra": balancer.extra,
    }


def _simple_member(member):
    return {
        "id": member.id,
        "ip": member.ip,
        "port": member.port,
        "balancer": _simple_balancer(member.balancer),
        "extra": member.extra,
    }



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import glob
import os.path
import tempfile

import pytest
import salt.modules.qemu_nbd as qemu_nbd
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class QemuNbdTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.qemu_nbd
    """

    def setup_loader_modules(self):
        return {qemu_nbd: {}}

    # 'connect' function tests: 1

    def test_connect(self):
        """
        Test if it activate nbd for an image file.
        """
        mock = MagicMock(return_value=True)
        with patch.dict(qemu_nbd.__salt__, {"cmd.run": mock}):
            with patch.object(os.path, "isfile", MagicMock(return_value=False)):
                self.assertEqual(qemu_nbd.connect("/tmp/image.raw"), "")
                self.assertEqual(qemu_nbd.connect("/tmp/image.raw"), "")

        with patch.object(os.path, "isfile", mock):
            with patch.object(glob, "glob", MagicMock(return_value=["/dev/nbd0"])):
                with patch.dict(
                    qemu_nbd.__salt__,
                    {"cmd.run": mock, "cmd.retcode": MagicMock(side_effect=[1, 0])},
                ):
                    self.assertEqual(qemu_nbd.connect("/tmp/image.raw"), "/dev/nbd0")

                with patch.dict(
                    qemu_nbd.__salt__,
                    {"cmd.run": mock, "cmd.retcode": MagicMock(return_value=False)},
                ):
                    self.assertEqual(qemu_nbd.connect("/tmp/image.raw"), "")

    # 'mount' function tests: 1

    def test_mount(self):
        """
        Test if it pass in the nbd connection device location,
        mount all partitions and return a dict of mount points.
        """
        mock = MagicMock(return_value=True)
        with patch.dict(qemu_nbd.__salt__, {"cmd.run": mock}):
            self.assertDictEqual(qemu_nbd.mount("/dev/nbd0"), {})

    # 'init' function tests: 1

    @pytest.mark.slow_test
    def test_init(self):
        """
        Test if it mount the named image via qemu-nbd
        and return the mounted roots
        """
        mock = MagicMock(return_value=True)
        with patch.dict(qemu_nbd.__salt__, {"cmd.run": mock}):
            self.assertEqual(qemu_nbd.init("/srv/image.qcow2"), "")

        with patch.object(os.path, "isfile", mock), patch.object(
            glob, "glob", MagicMock(return_value=["/dev/nbd0"])
        ), patch.dict(
            qemu_nbd.__salt__,
            {
                "cmd.run": mock,
                "mount.mount": mock,
                "cmd.retcode": MagicMock(side_effect=[1, 0]),
            },
        ):
            expected = {
                os.sep.join([tempfile.gettempdir(), "nbd", "nbd0", "nbd0"]): "/dev/nbd0"
            }
            self.assertDictEqual(qemu_nbd.init("/srv/image.qcow2"), expected)

    # 'clear' function tests: 1

    def test_clear(self):
        """
        Test if it pass in the mnt dict returned from nbd_mount
        to unmount and disconnect the image from nbd.
        """
        mock_run = MagicMock(return_value=True)
        with patch.dict(
            qemu_nbd.__salt__,
            {"cmd.run": mock_run, "mount.umount": MagicMock(side_effect=[False, True])},
        ):
            self.assertDictEqual(
                qemu_nbd.clear({"/mnt/foo": "/dev/nbd0p1"}), {"/mnt/foo": "/dev/nbd0p1"}
            )
            self.assertDictEqual(qemu_nbd.clear({"/mnt/foo": "/dev/nbd0p1"}), {})



import logging
import os
import shutil

import pytest
import salt.serializers.json as jsonserializer
import salt.serializers.msgpack as msgpackserializer
import salt.serializers.plist as plistserializer
import salt.serializers.python as pythonserializer
import salt.serializers.yaml as yamlserializer
import salt.states.file as filestate
from tests.support.mock import MagicMock, patch

log = logging.getLogger(__name__)


@pytest.fixture
def configure_loader_modules():
    return {
        filestate: {
            "__env__": "base",
            "__salt__": {"file.manage_file": False},
            "__serializers__": {
                "yaml.serialize": yamlserializer.serialize,
                "yaml.seserialize": yamlserializer.serialize,
                "python.serialize": pythonserializer.serialize,
                "json.serialize": jsonserializer.serialize,
                "plist.serialize": plistserializer.serialize,
                "msgpack.serialize": msgpackserializer.serialize,
            },
            "__opts__": {"test": False, "cachedir": ""},
            "__instance_id__": "",
            "__low__": {},
            "__utils__": {},
        }
    }


# 'rename' function tests: 1
def test_rename(tmp_path):
    """
    Test if the source file exists on the system,
    rename it to the named file.
    """
    name = str(tmp_path / "salt")
    source = str(tmp_path / "salt" / "salt")

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    comt = "Must provide name to file.rename"
    ret.update({"comment": comt, "name": ""})
    assert filestate.rename("", source) == ret

    mock_t = MagicMock(return_value=True)
    mock_f = MagicMock(return_value=False)

    mock_lex = MagicMock(side_effect=[False, True, True])
    with patch.object(os.path, "isabs", mock_f):
        comt = "Specified file {} is not an absolute path".format(name)
        ret.update({"comment": comt, "name": name})
        assert filestate.rename(name, source) == ret

    mock_lex = MagicMock(return_value=False)
    with patch.object(os.path, "isabs", mock_t):
        with patch.object(os.path, "lexists", mock_lex):
            comt = 'Source file "{}" has already been moved out of place'.format(source)
            ret.update({"comment": comt, "result": True})
            assert filestate.rename(name, source) == ret

    mock_lex = MagicMock(side_effect=[True, True, True])
    with patch.object(os.path, "isabs", mock_t):
        with patch.object(os.path, "lexists", mock_lex):
            comt = 'The target file "{}" exists and will not be overwritten'.format(
                name
            )
            ret.update({"comment": comt, "result": True})
            assert filestate.rename(name, source) == ret

    mock_lex = MagicMock(side_effect=[True, True, True])
    mock_rem = MagicMock(side_effect=IOError)
    with patch.object(os.path, "isabs", mock_t):
        with patch.object(os.path, "lexists", mock_lex):
            with patch.dict(filestate.__opts__, {"test": False}):
                comt = 'Failed to delete "{}" in preparation for forced move'.format(
                    name
                )
                with patch.dict(filestate.__salt__, {"file.remove": mock_rem}):
                    ret.update({"name": name, "comment": comt, "result": False})
                    assert filestate.rename(name, source, force=True) == ret

    mock_lex = MagicMock(side_effect=[True, False, False])
    with patch.object(os.path, "isabs", mock_t):
        with patch.object(os.path, "lexists", mock_lex):
            with patch.dict(filestate.__opts__, {"test": True}):
                comt = 'File "{}" is set to be moved to "{}"'.format(source, name)
                ret.update({"name": name, "comment": comt, "result": None})
                assert filestate.rename(name, source) == ret

    mock_lex = MagicMock(side_effect=[True, False, False])
    with patch.object(os.path, "isabs", mock_t):
        with patch.object(os.path, "lexists", mock_lex):
            with patch.object(os.path, "isdir", mock_f):
                with patch.dict(filestate.__opts__, {"test": False}):
                    comt = "The target directory {} is not present".format(tmp_path)
                    ret.update({"name": name, "comment": comt, "result": False})
                    assert filestate.rename(name, source) == ret

    mock_lex = MagicMock(side_effect=[True, False, False])
    with patch.object(os.path, "isabs", mock_t):
        with patch.object(os.path, "lexists", mock_lex):
            with patch.object(os.path, "isdir", mock_t):
                with patch.object(os.path, "islink", mock_f):
                    with patch.dict(filestate.__opts__, {"test": False}):
                        with patch.object(
                            shutil, "move", MagicMock(side_effect=IOError)
                        ):
                            comt = 'Failed to move "{}" to "{}"'.format(source, name)
                            ret.update({"name": name, "comment": comt, "result": False})
                            assert filestate.rename(name, source) == ret

    mock_lex = MagicMock(side_effect=[True, False, False])
    with patch.object(os.path, "isabs", mock_t):
        with patch.object(os.path, "lexists", mock_lex):
            with patch.object(os.path, "isdir", mock_t):
                with patch.object(os.path, "islink", mock_f):
                    with patch.dict(filestate.__opts__, {"test": False}):
                        with patch.object(shutil, "move", MagicMock()):
                            comt = 'Moved "{}" to "{}"'.format(source, name)
                            ret.update(
                                {
                                    "name": name,
                                    "comment": comt,
                                    "result": True,
                                    "changes": {name: source},
                                }
                            )
                            assert filestate.rename(name, source) == ret



"""
Module for editing power settings on macOS

 .. versionadded:: 2016.3.0
"""

import salt.utils.mac_utils
import salt.utils.platform
from salt.exceptions import SaltInvocationError

__virtualname__ = "power"


def __virtual__():
    """
    Only for macOS
    """
    if not salt.utils.platform.is_darwin():
        return (
            False,
            "The mac_power module could not be loaded: "
            "module only works on macOS systems.",
        )

    return __virtualname__


def _validate_sleep(minutes):
    """
    Helper function that validates the minutes parameter. Can be any number
    between 1 and 180. Can also be the string values "Never" and "Off".

    Because "On" and "Off" get converted to boolean values on the command line
    it will error if "On" is passed

    Returns: The value to be passed to the command
    """
    # Must be a value between 1 and 180 or Never/Off
    if isinstance(minutes, str):
        if minutes.lower() in ["never", "off"]:
            return "Never"
        else:
            msg = (
                "Invalid String Value for Minutes.\n"
                'String values must be "Never" or "Off".\n'
                "Passed: {}".format(minutes)
            )
            raise SaltInvocationError(msg)
    elif isinstance(minutes, bool):
        if minutes:
            msg = (
                "Invalid Boolean Value for Minutes.\n"
                'Boolean value "On" or "True" is not allowed.\n'
                'Salt CLI converts "On" to boolean True.\n'
                "Passed: {}".format(minutes)
            )
            raise SaltInvocationError(msg)
        else:
            return "Never"
    elif isinstance(minutes, int):
        if minutes in range(1, 181):
            return minutes
        else:
            msg = (
                "Invalid Integer Value for Minutes.\n"
                "Integer values must be between 1 and 180.\n"
                "Passed: {}".format(minutes)
            )
            raise SaltInvocationError(msg)
    else:
        msg = "Unknown Variable Type Passed for Minutes.\nPassed: {}".format(minutes)
        raise SaltInvocationError(msg)


def get_sleep():
    """
    Displays the amount of idle time until the machine sleeps. Settings for
    Computer, Display, and Hard Disk are displayed.

    :return: A dictionary containing the sleep status for Computer, Display, and
        Hard Disk

    :rtype: dict

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_sleep
    """
    return {
        "Computer": get_computer_sleep(),
        "Display": get_display_sleep(),
        "Hard Disk": get_harddisk_sleep(),
    }


def set_sleep(minutes):
    """
    Sets the amount of idle time until the machine sleeps. Sets the same value
    for Computer, Display, and Hard Disk. Pass "Never" or "Off" for computers
    that should never sleep.

    :param minutes: Can be an integer between 1 and 180 or "Never" or "Off"
    :ptype: int, str

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_sleep 120
        salt '*' power.set_sleep never
    """
    value = _validate_sleep(minutes)
    cmd = "systemsetup -setsleep {}".format(value)
    salt.utils.mac_utils.execute_return_success(cmd)

    state = []
    for check in (get_computer_sleep, get_display_sleep, get_harddisk_sleep):
        state.append(
            salt.utils.mac_utils.confirm_updated(
                value,
                check,
            )
        )
    return all(state)


def get_computer_sleep():
    """
    Display the amount of idle time until the computer sleeps.

    :return: A string representing the sleep settings for the computer
    :rtype: str

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_computer_sleep
    """
    ret = salt.utils.mac_utils.execute_return_result("systemsetup -getcomputersleep")
    return salt.utils.mac_utils.parse_return(ret)


def set_computer_sleep(minutes):
    """
    Set the amount of idle time until the computer sleeps. Pass "Never" of "Off"
    to never sleep.

    :param minutes: Can be an integer between 1 and 180 or "Never" or "Off"
    :ptype: int, str

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_computer_sleep 120
        salt '*' power.set_computer_sleep off
    """
    value = _validate_sleep(minutes)
    cmd = "systemsetup -setcomputersleep {}".format(value)
    salt.utils.mac_utils.execute_return_success(cmd)

    return salt.utils.mac_utils.confirm_updated(
        str(value),
        get_computer_sleep,
    )


def get_display_sleep():
    """
    Display the amount of idle time until the display sleeps.

    :return: A string representing the sleep settings for the displey
    :rtype: str

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_display_sleep
    """
    ret = salt.utils.mac_utils.execute_return_result("systemsetup -getdisplaysleep")
    return salt.utils.mac_utils.parse_return(ret)


def set_display_sleep(minutes):
    """
    Set the amount of idle time until the display sleeps. Pass "Never" of "Off"
    to never sleep.

    :param minutes: Can be an integer between 1 and 180 or "Never" or "Off"
    :ptype: int, str

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_display_sleep 120
        salt '*' power.set_display_sleep off
    """
    value = _validate_sleep(minutes)
    cmd = "systemsetup -setdisplaysleep {}".format(value)
    salt.utils.mac_utils.execute_return_success(cmd)

    return salt.utils.mac_utils.confirm_updated(
        str(value),
        get_display_sleep,
    )


def get_harddisk_sleep():
    """
    Display the amount of idle time until the hard disk sleeps.

    :return: A string representing the sleep settings for the hard disk
    :rtype: str

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_harddisk_sleep
    """
    ret = salt.utils.mac_utils.execute_return_result("systemsetup -getharddisksleep")
    return salt.utils.mac_utils.parse_return(ret)


def set_harddisk_sleep(minutes):
    """
    Set the amount of idle time until the harddisk sleeps. Pass "Never" of "Off"
    to never sleep.

    :param minutes: Can be an integer between 1 and 180 or "Never" or "Off"
    :ptype: int, str

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_harddisk_sleep 120
        salt '*' power.set_harddisk_sleep off
    """
    value = _validate_sleep(minutes)
    cmd = "systemsetup -setharddisksleep {}".format(value)
    salt.utils.mac_utils.execute_return_success(cmd)

    return salt.utils.mac_utils.confirm_updated(
        str(value),
        get_harddisk_sleep,
    )


def get_wake_on_modem():
    """
    Displays whether 'wake on modem' is on or off if supported

    :return: A string value representing the "wake on modem" settings
    :rtype: str

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_wake_on_modem
    """
    ret = salt.utils.mac_utils.execute_return_result("systemsetup -getwakeonmodem")
    return (
        salt.utils.mac_utils.validate_enabled(salt.utils.mac_utils.parse_return(ret))
        == "on"
    )


def set_wake_on_modem(enabled):
    """
    Set whether or not the computer will wake from sleep when modem activity is
    detected.

    :param bool enabled: True to enable, False to disable. "On" and "Off" are
        also acceptable values. Additionally you can pass 1 and 0 to represent
        True and False respectively

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_wake_on_modem True
    """
    state = salt.utils.mac_utils.validate_enabled(enabled)
    cmd = "systemsetup -setwakeonmodem {}".format(state)
    salt.utils.mac_utils.execute_return_success(cmd)

    return salt.utils.mac_utils.confirm_updated(
        state,
        get_wake_on_modem,
    )


def get_wake_on_network():
    """
    Displays whether 'wake on network' is on or off if supported

    :return: A string value representing the "wake on network" settings
    :rtype: string

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_wake_on_network
    """
    ret = salt.utils.mac_utils.execute_return_result(
        "systemsetup -getwakeonnetworkaccess"
    )
    return (
        salt.utils.mac_utils.validate_enabled(salt.utils.mac_utils.parse_return(ret))
        == "on"
    )


def set_wake_on_network(enabled):
    """
    Set whether or not the computer will wake from sleep when network activity
    is detected.

    :param bool enabled: True to enable, False to disable. "On" and "Off" are
        also acceptable values. Additionally you can pass 1 and 0 to represent
        True and False respectively

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_wake_on_network True
    """
    state = salt.utils.mac_utils.validate_enabled(enabled)
    cmd = "systemsetup -setwakeonnetworkaccess {}".format(state)
    salt.utils.mac_utils.execute_return_success(cmd)

    return salt.utils.mac_utils.confirm_updated(
        state,
        get_wake_on_network,
    )


def get_restart_power_failure():
    """
    Displays whether 'restart on power failure' is on or off if supported

    :return: A string value representing the "restart on power failure" settings
    :rtype: string

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_restart_power_failure
    """
    ret = salt.utils.mac_utils.execute_return_result(
        "systemsetup -getrestartpowerfailure"
    )
    return (
        salt.utils.mac_utils.validate_enabled(salt.utils.mac_utils.parse_return(ret))
        == "on"
    )


def set_restart_power_failure(enabled):
    """
    Set whether or not the computer will automatically restart after a power
    failure.

    :param bool enabled: True to enable, False to disable. "On" and "Off" are
        also acceptable values. Additionally you can pass 1 and 0 to represent
        True and False respectively

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_restart_power_failure True
    """
    state = salt.utils.mac_utils.validate_enabled(enabled)
    cmd = "systemsetup -setrestartpowerfailure {}".format(state)
    salt.utils.mac_utils.execute_return_success(cmd)

    return salt.utils.mac_utils.confirm_updated(
        state,
        get_restart_power_failure,
    )


def get_restart_freeze():
    """
    Displays whether 'restart on freeze' is on or off if supported

    :return: A string value representing the "restart on freeze" settings
    :rtype: string

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_restart_freeze
    """
    ret = salt.utils.mac_utils.execute_return_result("systemsetup -getrestartfreeze")
    return (
        salt.utils.mac_utils.validate_enabled(salt.utils.mac_utils.parse_return(ret))
        == "on"
    )


def set_restart_freeze(enabled):
    """
    Specifies whether the server restarts automatically after a system freeze.
    This setting doesn't seem to be editable. The command completes successfully
    but the setting isn't actually updated. This is probably a macOS. The
    functions remains in case they ever fix the bug.

    :param bool enabled: True to enable, False to disable. "On" and "Off" are
        also acceptable values. Additionally you can pass 1 and 0 to represent
        True and False respectively

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_restart_freeze True
    """
    state = salt.utils.mac_utils.validate_enabled(enabled)
    cmd = "systemsetup -setrestartfreeze {}".format(state)
    salt.utils.mac_utils.execute_return_success(cmd)

    return salt.utils.mac_utils.confirm_updated(state, get_restart_freeze, True)


def get_sleep_on_power_button():
    """
    Displays whether 'allow power button to sleep computer' is on or off if
    supported

    :return: A string value representing the "allow power button to sleep
        computer" settings

    :rtype: string

    CLI Example:

    .. code-block:: bash

        salt '*' power.get_sleep_on_power_button
    """
    ret = salt.utils.mac_utils.execute_return_result(
        "systemsetup -getallowpowerbuttontosleepcomputer"
    )
    return (
        salt.utils.mac_utils.validate_enabled(salt.utils.mac_utils.parse_return(ret))
        == "on"
    )


def set_sleep_on_power_button(enabled):
    """
    Set whether or not the power button can sleep the computer.

    :param bool enabled: True to enable, False to disable. "On" and "Off" are
        also acceptable values. Additionally you can pass 1 and 0 to represent
        True and False respectively

    :return: True if successful, False if not
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' power.set_sleep_on_power_button True
    """
    state = salt.utils.mac_utils.validate_enabled(enabled)
    cmd = "systemsetup -setallowpowerbuttontosleepcomputer {}".format(state)
    salt.utils.mac_utils.execute_return_success(cmd)

    return salt.utils.mac_utils.confirm_updated(
        state,
        get_sleep_on_power_button,
    )



"""
    :codeauthor: Mike Place <mp@saltstack.com>
"""


import time
from collections import namedtuple

import salt.modules.ps as ps
import salt.utils.data
import salt.utils.psutil_compat as psutil
from tests.support.mock import MagicMock, Mock, call, patch
from tests.support.unit import TestCase, skipIf

HAS_PSUTIL_VERSION = False


PSUTIL2 = psutil.version_info >= (2, 0)

STUB_CPU_TIMES = namedtuple("cputimes", "user nice system idle")(1, 2, 3, 4)
STUB_VIRT_MEM = namedtuple("vmem", "total available percent used free")(
    1000, 500, 50, 500, 500
)
STUB_SWAP_MEM = namedtuple("swap", "total used free percent sin sout")(
    1000, 500, 500, 50, 0, 0
)
STUB_PHY_MEM_USAGE = namedtuple("usage", "total used free percent")(1000, 500, 500, 50)
STUB_DISK_PARTITION = namedtuple("partition", "device mountpoint fstype, opts")(
    "/dev/disk0s2", "/", "hfs", "rw,local,rootfs,dovolfs,journaled,multilabel"
)
STUB_DISK_USAGE = namedtuple("usage", "total used free percent")(1000, 500, 500, 50)
STUB_NETWORK_IO = namedtuple(
    "iostat",
    "bytes_sent, bytes_recv, packets_sent, packets_recv, errin errout dropin dropout",
)(1000, 2000, 500, 600, 1, 2, 3, 4)
STUB_DISK_IO = namedtuple(
    "iostat", "read_count, write_count, read_bytes, write_bytes, read_time, write_time"
)(1000, 2000, 500, 600, 2000, 3000)
STUB_USER = namedtuple("user", "name, terminal, host, started")(
    "bdobbs", "ttys000", "localhost", 0.0
)
if psutil.version_info >= (0, 6, 0):
    HAS_PSUTIL_VERSION = True

STUB_PID_LIST = [0, 1, 2, 3]

try:
    import utmp  # pylint: disable=W0611

    HAS_UTMP = True
except ImportError:
    HAS_UTMP = False
# pylint: enable=import-error,unused-import


def _get_proc_name(proc):
    return proc.name() if PSUTIL2 else proc.name


def _get_proc_pid(proc):
    return proc.pid


class DummyProcess:
    """
    Dummy class to emulate psutil.Process. This ensures that _any_ string
    values used for any of the options passed in are converted to str types on
    both Python 2 and Python 3.
    """

    def __init__(
        self,
        cmdline=None,
        create_time=None,
        name=None,
        status=None,
        username=None,
        pid=None,
    ):
        self._cmdline = salt.utils.data.decode(
            cmdline if cmdline is not None else [], to_str=True
        )
        self._create_time = salt.utils.data.decode(
            create_time if create_time is not None else time.time(), to_str=True
        )
        self._name = salt.utils.data.decode(
            name if name is not None else [], to_str=True
        )
        self._status = salt.utils.data.decode(status, to_str=True)
        self._username = salt.utils.data.decode(username, to_str=True)
        self._pid = salt.utils.data.decode(
            pid if pid is not None else 12345, to_str=True
        )

    def cmdline(self):
        return self._cmdline

    def create_time(self):
        return self._create_time

    def name(self):
        return self._name

    def status(self):
        return self._status

    def username(self):
        return self._username

    def pid(self):
        return self._pid


class PsTestCase(TestCase):
    def setUp(self):
        self.mocked_proc = mocked_proc = MagicMock("salt.utils.psutil_compat.Process")
        if PSUTIL2:
            self.mocked_proc.name = Mock(return_value="test_mock_proc")
            self.mocked_proc.pid = Mock(return_value=9999999999)
        else:
            self.mocked_proc.name = "test_mock_proc"
            self.mocked_proc.pid = 9999999999

    @skipIf(not ps.PSUTIL2, "Only run for psutil 2.x")
    def test__get_proc_cmdline(self):
        cmdline = ["echo", "питон"]
        ret = ps._get_proc_cmdline(DummyProcess(cmdline=cmdline))
        assert ret == cmdline, ret

    def test_get_pid_list(self):
        with patch(
            "salt.utils.psutil_compat.pids", MagicMock(return_value=STUB_PID_LIST)
        ):
            self.assertListEqual(STUB_PID_LIST, ps.get_pid_list())

    def test_kill_pid(self):
        with patch("salt.utils.psutil_compat.Process") as send_signal_mock:
            ps.kill_pid(0, signal=999)
            self.assertEqual(send_signal_mock.call_args, call(0))

    def test_pkill(self):
        with patch("salt.utils.psutil_compat.Process.send_signal"), patch(
            "salt.utils.psutil_compat.process_iter",
            MagicMock(return_value=[self.mocked_proc]),
        ):
            self.mocked_proc.send_signal = MagicMock()
            test_signal = 1234
            ps.pkill(_get_proc_name(self.mocked_proc), signal=test_signal)
            self.assertEqual(self.mocked_proc.send_signal.call_args, call(test_signal))

    def test_pgrep(self):
        with patch(
            "salt.utils.psutil_compat.process_iter",
            MagicMock(return_value=[self.mocked_proc]),
        ):
            self.assertIn(
                _get_proc_pid(self.mocked_proc),
                ps.pgrep(_get_proc_name(self.mocked_proc)),
            )

    def test_pgrep_regex(self):
        with patch(
            "salt.utils.psutil_compat.process_iter",
            MagicMock(return_value=[self.mocked_proc]),
        ):
            self.assertIn(
                _get_proc_pid(self.mocked_proc),
                ps.pgrep("t.st_[a-z]+_proc", pattern_is_regex=True),
            )

    def test_cpu_percent(self):
        with patch("salt.utils.psutil_compat.cpu_percent", MagicMock(return_value=1)):
            self.assertEqual(ps.cpu_percent(), 1)

    def test_cpu_times(self):
        with patch(
            "salt.utils.psutil_compat.cpu_times", MagicMock(return_value=STUB_CPU_TIMES)
        ):
            self.assertDictEqual(
                {"idle": 4, "nice": 2, "system": 3, "user": 1}, ps.cpu_times()
            )

    @skipIf(
        HAS_PSUTIL_VERSION is False, "psutil 0.6.0 or greater is required for this test"
    )
    def test_virtual_memory(self):
        with patch(
            "salt.utils.psutil_compat.virtual_memory",
            MagicMock(return_value=STUB_VIRT_MEM),
        ):
            self.assertDictEqual(
                {
                    "used": 500,
                    "total": 1000,
                    "available": 500,
                    "percent": 50,
                    "free": 500,
                },
                ps.virtual_memory(),
            )

    @skipIf(
        HAS_PSUTIL_VERSION is False, "psutil 0.6.0 or greater is required for this test"
    )
    def test_swap_memory(self):
        with patch(
            "salt.utils.psutil_compat.swap_memory",
            MagicMock(return_value=STUB_SWAP_MEM),
        ):
            self.assertDictEqual(
                {
                    "used": 500,
                    "total": 1000,
                    "percent": 50,
                    "free": 500,
                    "sin": 0,
                    "sout": 0,
                },
                ps.swap_memory(),
            )

    def test_disk_partitions(self):
        with patch(
            "salt.utils.psutil_compat.disk_partitions",
            MagicMock(return_value=[STUB_DISK_PARTITION]),
        ):
            self.assertDictEqual(
                {
                    "device": "/dev/disk0s2",
                    "mountpoint": "/",
                    "opts": "rw,local,rootfs,dovolfs,journaled,multilabel",
                    "fstype": "hfs",
                },
                ps.disk_partitions()[0],
            )

    def test_disk_usage(self):
        with patch(
            "salt.utils.psutil_compat.disk_usage",
            MagicMock(return_value=STUB_DISK_USAGE),
        ):
            self.assertDictEqual(
                {"used": 500, "total": 1000, "percent": 50, "free": 500},
                ps.disk_usage("DUMMY_PATH"),
            )

    def test_disk_partition_usage(self):
        with patch(
            "salt.utils.psutil_compat.disk_partitions",
            MagicMock(return_value=[STUB_DISK_PARTITION]),
        ):
            self.assertDictEqual(
                {
                    "device": "/dev/disk0s2",
                    "mountpoint": "/",
                    "opts": "rw,local,rootfs,dovolfs,journaled,multilabel",
                    "fstype": "hfs",
                },
                ps.disk_partitions()[0],
            )

    def test_network_io_counters(self):
        with patch(
            "salt.utils.psutil_compat.net_io_counters",
            MagicMock(return_value=STUB_NETWORK_IO),
        ):
            self.assertDictEqual(
                {
                    "packets_sent": 500,
                    "packets_recv": 600,
                    "bytes_recv": 2000,
                    "dropout": 4,
                    "bytes_sent": 1000,
                    "errout": 2,
                    "errin": 1,
                    "dropin": 3,
                },
                ps.network_io_counters(),
            )

    def test_disk_io_counters(self):
        with patch(
            "salt.utils.psutil_compat.disk_io_counters",
            MagicMock(return_value=STUB_DISK_IO),
        ):
            self.assertDictEqual(
                {
                    "read_time": 2000,
                    "write_bytes": 600,
                    "read_bytes": 500,
                    "write_time": 3000,
                    "read_count": 1000,
                    "write_count": 2000,
                },
                ps.disk_io_counters(),
            )

    def test_get_users(self):
        with patch(
            "salt.utils.psutil_compat.users", MagicMock(return_value=[STUB_USER])
        ):
            self.assertDictEqual(
                {
                    "terminal": "ttys000",
                    "started": 0.0,
                    "host": "localhost",
                    "name": "bdobbs",
                },
                ps.get_users()[0],
            )

    def test_top(self):
        """
        See the following issue:

        https://github.com/saltstack/salt/issues/56942
        """
        # Limiting to one process because the test suite might be running as
        # PID 1 under docker and there may only *be* one process running.
        result = ps.top(num_processes=1, interval=0)
        assert len(result) == 1

    def test_top_zombie_process(self):
        # Get 3 pids that are currently running on the system
        pids = psutil.pids()[:3]
        # Get a process instance for each of the pids
        processes = [psutil.Process(pid) for pid in pids]

        # Patch the middle process to raise ZombieProcess when .cpu_times is called
        def raise_exception():
            raise psutil.ZombieProcess(processes[1].pid)

        processes[1].cpu_times = raise_exception

        # Make sure psutil.pids only returns the above 3 pids
        with patch("salt.utils.psutil_compat.pids", return_value=pids):
            # Make sure we use our process list from above
            with patch("salt.utils.psutil_compat.Process", side_effect=processes):
                result = ps.top(num_processes=1, interval=0)
                assert len(result) == 1

    ## This is commented out pending discussion on https://github.com/saltstack/salt/commit/2e5c3162ef87cca8a2c7b12ade7c7e1b32028f0a
    # @skipIf(not HAS_UTMP, "The utmp module must be installed to run test_get_users_utmp()")
    # @patch('salt.utils.psutil_compat.get_users', new=MagicMock(return_value=None))  # This will force the function to use utmp
    # def test_get_users_utmp(self):
    #     pass



import salt.utils.platform
import salt.utils.win_functions as win_functions
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

try:
    import win32net

    HAS_WIN32 = True

    class WinError(win32net.error):
        winerror = 0


except ImportError:
    HAS_WIN32 = False


class WinFunctionsTestCase(TestCase):
    """
    Test cases for salt.utils.win_functions.
    """

    def test_escape_argument_simple(self):
        """
        Test to make sure we encode simple arguments correctly
        """
        encoded = win_functions.escape_argument("simple")

        self.assertEqual(encoded, "simple")

    def test_escape_argument_with_space(self):
        """
        Test to make sure we encode arguments containing spaces correctly
        """
        encoded = win_functions.escape_argument("with space")

        self.assertEqual(encoded, '^"with space^"')

    def test_escape_argument_simple_path(self):
        """
        Test to make sure we encode simple path arguments correctly
        """
        encoded = win_functions.escape_argument("C:\\some\\path")

        self.assertEqual(encoded, "C:\\some\\path")

    def test_escape_argument_path_with_space(self):
        """
        Test to make sure we encode path arguments containing spaces correctly
        """
        encoded = win_functions.escape_argument("C:\\Some Path\\With Spaces")

        self.assertEqual(encoded, '^"C:\\Some Path\\With Spaces^"')

    @skipIf(not salt.utils.platform.is_windows(), "WinDLL only available on Windows")
    def test_broadcast_setting_change(self):
        """
        Test to rehash the Environment variables
        """
        self.assertTrue(win_functions.broadcast_setting_change())

    @skipIf(not salt.utils.platform.is_windows(), "WinDLL only available on Windows")
    def test_get_user_groups(self):
        groups = ["Administrators", "Users"]
        with patch("win32net.NetUserGetLocalGroups", return_value=groups):
            ret = win_functions.get_user_groups("Administrator")
            self.assertListEqual(groups, ret)

    @skipIf(not salt.utils.platform.is_windows(), "WinDLL only available on Windows")
    def test_get_user_groups_sid(self):
        groups = ["Administrators", "Users"]
        expected = ["S-1-5-32-544", "S-1-5-32-545"]
        with patch("win32net.NetUserGetLocalGroups", return_value=groups):
            ret = win_functions.get_user_groups("Administrator", sid=True)
            self.assertListEqual(expected, ret)

    @skipIf(not salt.utils.platform.is_windows(), "WinDLL only available on Windows")
    def test_get_user_groups_system(self):
        groups = ["SYSTEM"]
        with patch("win32net.NetUserGetLocalGroups", return_value=groups):
            ret = win_functions.get_user_groups("SYSTEM")
            self.assertListEqual(groups, ret)

    @skipIf(not salt.utils.platform.is_windows(), "WinDLL only available on Windows")
    @skipIf(not HAS_WIN32, "Requires pywin32 libraries")
    def test_get_user_groups_unavailable_dc(self):
        groups = ["Administrators", "Users"]
        win_error = WinError()
        win_error.winerror = 1722
        effect = [win_error, groups]
        with patch("win32net.NetUserGetLocalGroups", side_effect=effect):
            ret = win_functions.get_user_groups("Administrator")
            self.assertListEqual(groups, ret)

    @skipIf(not salt.utils.platform.is_windows(), "WinDLL only available on Windows")
    @skipIf(not HAS_WIN32, "Requires pywin32 libraries")
    def test_get_user_groups_unknown_dc(self):
        groups = ["Administrators", "Users"]
        win_error = WinError()
        win_error.winerror = 2453
        effect = [win_error, groups]
        with patch("win32net.NetUserGetLocalGroups", side_effect=effect):
            ret = win_functions.get_user_groups("Administrator")
            self.assertListEqual(groups, ret)

    @skipIf(not salt.utils.platform.is_windows(), "WinDLL only available on Windows")
    @skipIf(not HAS_WIN32, "Requires pywin32 libraries")
    def test_get_user_groups_missing_permission(self):
        groups = ["Administrators", "Users"]
        win_error = WinError()
        win_error.winerror = 5
        effect = [win_error, groups]
        with patch("win32net.NetUserGetLocalGroups", side_effect=effect):
            ret = win_functions.get_user_groups("Administrator")
            self.assertListEqual(groups, ret)

    @skipIf(not salt.utils.platform.is_windows(), "WinDLL only available on Windows")
    @skipIf(not HAS_WIN32, "Requires pywin32 libraries")
    def test_get_user_groups_error(self):
        win_error = WinError()
        win_error.winerror = 1927
        mock_error = MagicMock(side_effect=win_error)
        with patch("win32net.NetUserGetLocalGroups", side_effect=mock_error):
            with self.assertRaises(WinError):
                win_functions.get_user_groups("Administrator")



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import salt.modules.system as system
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SystemTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.system
    """

    def setup_loader_modules(self):
        return {system: {}}

    def test_halt(self):
        """
        Test to halt a running system
        """
        with patch.dict(system.__salt__, {"cmd.run": MagicMock(return_value="A")}):
            self.assertEqual(system.halt(), "A")

    def test_init(self):
        """
        Test to change the system runlevel on sysV compatible systems
        """
        with patch.dict(system.__salt__, {"cmd.run": MagicMock(return_value="A")}):
            self.assertEqual(system.init("r"), "A")

    def test_poweroff(self):
        """
        Test to poweroff a running system
        """
        with patch.dict(system.__salt__, {"cmd.run": MagicMock(return_value="A")}):
            self.assertEqual(system.poweroff(), "A")

    def test_reboot(self):
        """
        Test to reboot the system with shutdown -r
        """
        cmd_mock = MagicMock(return_value="A")
        with patch.dict(system.__salt__, {"cmd.run": cmd_mock}):
            self.assertEqual(system.reboot(), "A")
        cmd_mock.assert_called_with(["shutdown", "-r", "now"], python_shell=False)

    def test_reboot_with_delay(self):
        """
        Test to reboot the system using shutdown -r with a delay
        """
        cmd_mock = MagicMock(return_value="A")
        with patch.dict(system.__salt__, {"cmd.run": cmd_mock}):
            self.assertEqual(system.reboot(at_time=5), "A")
        cmd_mock.assert_called_with(["shutdown", "-r", "5"], python_shell=False)

    def test_shutdown(self):
        """
        Test to shutdown a running system
        """
        cmd_mock = MagicMock(return_value="A")
        with patch.dict(system.__salt__, {"cmd.run": cmd_mock}), patch(
            "salt.utils.platform.is_freebsd", MagicMock(return_value=False)
        ), patch("salt.utils.platform.is_netbsd", MagicMock(return_value=False)), patch(
            "salt.utils.platform.is_openbsd", MagicMock(return_value=False)
        ):
            self.assertEqual(system.shutdown(), "A")
        cmd_mock.assert_called_with(["shutdown", "-h", "now"], python_shell=False)

    def test_shutdown_freebsd(self):
        """
        Test to shutdown a running FreeBSD system
        """
        cmd_mock = MagicMock(return_value="A")
        with patch.dict(system.__salt__, {"cmd.run": cmd_mock}), patch(
            "salt.utils.platform.is_freebsd", MagicMock(return_value=True)
        ), patch("salt.utils.platform.is_netbsd", MagicMock(return_value=False)), patch(
            "salt.utils.platform.is_openbsd", MagicMock(return_value=False)
        ):
            self.assertEqual(system.shutdown(), "A")
        cmd_mock.assert_called_with(["shutdown", "-p", "now"], python_shell=False)

    def test_shutdown_netbsd(self):
        """
        Test to shutdown a running NetBSD system
        """
        cmd_mock = MagicMock(return_value="A")
        with patch.dict(system.__salt__, {"cmd.run": cmd_mock}), patch(
            "salt.utils.platform.is_freebsd", MagicMock(return_value=False)
        ), patch("salt.utils.platform.is_netbsd", MagicMock(return_value=True)), patch(
            "salt.utils.platform.is_openbsd", MagicMock(return_value=False)
        ):
            self.assertEqual(system.shutdown(), "A")
        cmd_mock.assert_called_with(["shutdown", "-p", "now"], python_shell=False)

    def test_shutdown_openbsd(self):
        """
        Test to shutdown a running OpenBSD system
        """
        cmd_mock = MagicMock(return_value="A")
        with patch.dict(system.__salt__, {"cmd.run": cmd_mock}), patch(
            "salt.utils.platform.is_freebsd", MagicMock(return_value=False)
        ), patch("salt.utils.platform.is_netbsd", MagicMock(return_value=False)), patch(
            "salt.utils.platform.is_openbsd", MagicMock(return_value=True)
        ):
            self.assertEqual(system.shutdown(), "A")
        cmd_mock.assert_called_with(["shutdown", "-p", "now"], python_shell=False)



import pytest
import salt.wheel
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.unit import TestCase


@pytest.mark.windows_whitelisted
@pytest.mark.usefixtures("salt_sub_minion")
class KeyWheelModuleTest(TestCase, AdaptedConfigurationTestCaseMixin):
    def setUp(self):
        self.wheel = salt.wheel.Wheel(dict(self.get_config("client_config")))

    def tearDown(self):
        del self.wheel

    @pytest.mark.slow_test
    def test_list_all(self):
        ret = self.wheel.cmd("key.list_all", print_event=False)
        for host in ["minion", "sub_minion"]:
            self.assertIn(host, ret["minions"])

    @pytest.mark.slow_test
    def test_gen(self):
        ret = self.wheel.cmd(
            "key.gen", kwarg={"id_": "soundtechniciansrock"}, print_event=False
        )

        self.assertIn("pub", ret)
        self.assertIn("priv", ret)
        try:
            self.assertTrue(ret.get("pub", "").startswith("-----BEGIN PUBLIC KEY-----"))
        except AssertionError:
            self.assertTrue(
                ret.get("pub", "").startswith("-----BEGIN RSA PUBLIC KEY-----")
            )

        self.assertTrue(
            ret.get("priv", "").startswith("-----BEGIN RSA PRIVATE KEY-----")
        )

    @pytest.mark.slow_test
    def test_master_key_str(self):
        ret = self.wheel.cmd("key.master_key_str", print_event=False)
        self.assertIn("local", ret)
        self.assertIn("master.pub", ret.get("local"))
        self.assertTrue(
            ret.get("local").get("master.pub").startswith("-----BEGIN PUBLIC KEY-----")
        )
